/* 
 * Copyright (C) 2002, 2003 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "check.h"


#define LIST_LABEL_MAX_LEN 30
#define PROGRESS_LABEL_MAX_LEN 50

#define UC_CHECK_IO_WHILE_CONDITION \
	( \
        	(!uc_check_gdk_input_end) && \
        	(!uc_check_cancel_value) && \
        	(!uc_check_ignore_item_value) && \
        	(!uc_check_timeout_flag) \
	)

#define UC_CHECK_PARSER_WHILE_CONDITION \
	( \
		begin && \
		((strlen (begin) > SEARCH_STRING_MAX_LEN + 1)) && \
		(!uc_check_cancel_value) && \
		(!uc_check_ignore_item_value) \
	)


typedef struct _UCGetContentReadData
{
  GString *string;
  guint content_length;
  guint header_length;
}
UCGetContentReadData;


struct
{
  UCLinkType type;
  gchar *icon_file;
}


static uc_check_link_types[] = {
  {LINK_TYPE_HREF, UC_PIXMAPS_DIR "/uc_treeview_href.png"},
  {LINK_TYPE_IMAGE, UC_PIXMAPS_DIR "/uc_treeview_img.png"},
  {LINK_TYPE_NONE, UC_PIXMAPS_DIR "/uc_treeview_unknown.png"}
};


/* structure to store links properties */
typedef struct
{
  guint uid;
  UCLinkType link_type;
  GdkPixbuf *link_icon;
  gchar *url_original;
  gchar *url;
  gchar *label;
  gchar *protocol;
  gchar *h_name;
  gchar *port;
  gchar *path;
  gchar *file;
  gchar *args;
  gchar *h_address;
  GHashTable *header;
  guint header_size;
  guint depth_level;
  gboolean is_parsable;
  GList *childs;
}
UCLinkProperties;


/* struct to point on the current url or host elements
 * (for timeouts callback) */
static struct _UCCurrentItem
{
  GList *list_ptr;
  gchar *host;
  gchar *url;
}
currentItem;


typedef struct
{
  UCLinkType type;
  gchar *label;
  gchar *value;
}
UCTagLinkProperties;

/* global variable to know when a network operation is accomplished */
static gboolean uc_check_gdk_input_end = FALSE;


/* temporary list of already checked urls */
static GList *uc_check_tmp_list = NULL;


/* unique id for a glist properties node */
static guint uc_check_node_uid = 1;


static gboolean uc_check_timeout_flag = FALSE;


/* if FALSE, abort the check */
static gboolean uc_check_cancel_value = FALSE;


/* if FALSE, abort the check of current url*/
static gboolean uc_check_ignore_item_value = FALSE;


/* --FIXME -- */
static gchar *informations_buffer = NULL;


static GList *uc_check_buttons_get_list (const GtkWidget * widget);
static void uc_check_display_informations_set_label (gpointer key,
						     gpointer value,
						     gpointer user_data);
static void uc_check_right_frames_reset (void);
static void uc_check_insert_badlink (void);
static UCLinkProperties *uc_check_search_link_properties_list (GList * list,
							       const guint
							       id);
static void uc_check_free_link_properties_list_hash_table (gpointer key,
							   gpointer value,
							   gpointer
							   user_data);
static GList *uc_check_url_get_links_real (UCLinkProperties * url,
					   const gchar * host,
					   const guint depth);
static GHashTable *uc_check_url_get_header_timeout (void);
static GHashTable *uc_check_url_get_header_ignored (void);
static void uc_check_free_properties_node (UCLinkProperties * prop);
static gchar *uc_check_url_get_content (UCLinkProperties * prop,
					const gchar * current_path,
					const gchar * current_file,
					const gchar * current_args,
					gchar * current_host);
static gchar *uc_check_url_get_content_bis (UCLinkProperties * prop);
static gboolean uc_check_url_in_checked_links (gchar * url);
static GList *uc_check_url_get_links (const gchar * host,
				      UCLinkProperties * url,
				      const guint depth, GList * list_all);
static GHashTable *uc_check_url_get_header (UCLinkProperties * prop);
static void uc_check_display_state_message (const gint label_pos,
					    const gchar * message);
static gboolean uc_check_content_type_accepted (const gchar * content_type);
static void uc_check_currentitem_set_list (gpointer * list);
static gboolean uc_check_protocol_accepted (gchar * protocol);
static void uc_check_wait (void);
static UCLinkProperties *uc_check_link_get_properties (const gchar *
						       current_host,
						       const gchar *
						       current_path,
						       gchar * url);
static gchar *uc_check_format_list_label (gchar * label);
static UCTagLinkProperties *uc_check_get_tag_link_properties (gchar * buffer);
static void uc_check_free_tag_link_properties_node (UCTagLinkProperties *
						    prop);
static GdkPixbuf *uc_check_get_link_icon (const UCLinkType link_type);

static void
uc_check_gdk_input_get_content_read (gpointer data, gint fd,
				     GdkInputCondition in);
static void uc_check_gdk_input_get_content_send (gpointer buffer, gint fd,
						 GdkInputCondition in);

static void uc_check_gdk_input_get_header_read (gpointer data, gint fd,
						GdkInputCondition in);
static void uc_check_gdk_input_get_header_send (gpointer buffer, gint fd,
						GdkInputCondition in);
static void uc_check_progress_bar_set_percent (gfloat percent);
static void uc_check_progress_bar_set_pulse (void);
static gchar *uc_check_format_progress_dialog_label (const gchar * label);



/*
 * set the timeout flag
 */
void
uc_check_timeout_flag_set (const gboolean flag)
{
  uc_check_timeout_flag = flag;
}


/*
 * get the timeout flag
 */
gboolean
uc_check_timeout_flag_get ()
{
  return uc_check_timeout_flag;
}


/*
 * return the label for a given link type
 */
static GdkPixbuf *
uc_check_get_link_icon (const UCLinkType link_type)
{
  guint i = 0;

  for (i = 0; i < LINK_TYPES_LEN; i++)
    if (uc_check_link_types[i].type == link_type)
      return gdk_pixbuf_new_from_file (uc_check_link_types[i].icon_file,
				       NULL);

  return gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/uc_treeview_unknown.png",
				   NULL);
}


/*
 * return a formated label to display in the
 * url tree view list
 */
static gchar *
uc_check_format_list_label (gchar * label)
{
  gchar *str = NULL;
  gchar *tmp = NULL;

  if (label)
    g_strstrip (label);

  if ((!label) || (!strlen (label)))
    tmp = g_strdup (_("<no label>"));
  else if (strlen (label) > ((LIST_LABEL_MAX_LEN - 5) + 1))
    {
      gchar *tmp_cut = NULL;

      tmp_cut = g_memdup (label, (LIST_LABEL_MAX_LEN - 5) + 1);
      tmp_cut[LIST_LABEL_MAX_LEN - 5] = '\0';
      tmp = g_strconcat (tmp_cut, "[...]", NULL);

      g_free (tmp_cut);
    }
  else
    tmp = g_strdup (label);

  if (!g_utf8_validate (tmp, strlen (tmp), NULL))
    str = g_convert (tmp, strlen (tmp), "UTF-8", "LATIN1", NULL, NULL, NULL);
  else
    str = g_strdup (tmp);

  g_free (tmp);

  return str;
}


/*
 * return a formated label to display in the
 * progress dialog box
 */
static gchar *
uc_check_format_progress_dialog_label (const gchar * label)
{
  gchar *str = NULL;
  gchar *tmp = NULL;

  if (strlen (label) > (PROGRESS_LABEL_MAX_LEN - 5))
    {
      gchar *tmp_pre = NULL;
      gchar *tmp_post = NULL;
      guint cut_point = (PROGRESS_LABEL_MAX_LEN / 2);

      tmp_pre = g_memdup (label, (cut_point - 3) + 1);
      tmp_pre[cut_point - 3] = '\0';
      tmp_post =
	g_memdup (&label[strlen (label) - (cut_point - 2) + 1],
		  (cut_point + 2) + 1);
      tmp_post[cut_point + 2] = '\0';
      tmp = g_strconcat (tmp_pre, "[...]", tmp_post, NULL);

      g_free (tmp_pre);
      g_free (tmp_post);
    }
  else
    tmp = g_strdup (label);

  if (!g_utf8_validate (tmp, strlen (tmp), NULL))
    str = g_convert (tmp, strlen (tmp), "UTF-8", "LATIN1", NULL, NULL, NULL);
  else
    str = g_strdup (tmp);

  g_free (tmp);

  return str;
}

/*
 * get the source of a given page
 */
static gchar *
uc_check_url_get_content (UCLinkProperties * prop,
			  const gchar * current_path,
			  const gchar * current_file,
			  const gchar * current_args, gchar * current_host)
{
  gchar buffer[BUFFER_LEN + 1] = { 0 };
  guint sock = 0;
  guint gifd = 0;
  guint content_length = 0;
  gchar *content_length_char = NULL;
  gchar *str = NULL;
  UCGetContentReadData read_data;

  if ((sock =
       uc_utils_socket_open (current_host, atoi (URL_DEFAULT_PORT),
			     UC_SOCKET_OPEN_DEFAULT)) <= 0)
    return NULL;

  snprintf (buffer, BUFFER_LEN, "GET %s://%s%s%s%s HTTP/1.0\r\n"
	    "Host: %s\r\n"
	    "User-agent: %s\r\n"
	    "Referer: %s\r\n"
	    "\r\n",
	    prop->protocol,
	    current_host,
	    current_path,
	    current_file, current_args, current_host, USER_AGENT, REFERER);

  uc_check_display_state_message (CHECK_MESSAGE_LABEL_SECOND, prop->url);
  uc_check_display_state_message (CHECK_MESSAGE_LABEL_THIRD,
				  _("Retrieving page source..."));
  uc_application_update_ui ();

  /* process the sending in background */
  uc_check_gdk_input_end = FALSE;
  gifd =
    gdk_input_add (sock, GDK_INPUT_WRITE,
		   (GdkInputFunction) uc_check_gdk_input_get_content_send,
		   &buffer);

  /* wait for the end of the sending operation */
  while (UC_CHECK_IO_WHILE_CONDITION)
    uc_application_update_ui ();

  gdk_input_remove (gifd);

  /* quit if a timeout occured */
  if (uc_check_timeout_flag)
    {
      close (sock);
      return NULL;
    }

  if ((content_length_char =
       g_hash_table_lookup (prop->header, CONTENT_LENGTH)))
    {
      if (strlen (content_length_char))
	content_length = atoi (content_length_char);
    }
  else
    content_length = 0;

  /* process the reading in background */
  read_data.string = g_string_new ("");
  read_data.header_length = prop->header_size;
  read_data.content_length = content_length;
  uc_check_gdk_input_end = FALSE;
  gifd =
    gdk_input_add (sock, GDK_INPUT_READ,
		   (GdkInputFunction) uc_check_gdk_input_get_content_read,
		   &read_data);

  /* wait for the end of the reading operation */
  while (UC_CHECK_IO_WHILE_CONDITION)
    uc_application_update_ui ();

  gdk_input_remove (gifd);

  /* quit if a timeout occured */
  if (uc_check_timeout_flag || (!read_data.string->len))
    {
      g_string_free (read_data.string, TRUE);
      close (sock);
      return NULL;
    }

  close (sock);

  str = read_data.string->str;
  g_string_free (read_data.string, FALSE);

  return str;
}


/*
 * free a UCLinkProperties node
 */
void static
uc_check_free_properties_node (UCLinkProperties * prop)
{
  if (!prop)
    return;

  g_free (prop->label);
  g_free (prop->url);
  g_free (prop->protocol);
  g_free (prop->h_name);
  g_free (prop->port);
  g_free (prop->path);
  g_free (prop->file);
  g_free (prop->args);
  g_free (prop->h_address);
  g_free (prop->url_original);

  if (prop->header != NULL)
    {
      g_hash_table_foreach (prop->header,
			    uc_check_free_link_properties_list_hash_table,
			    NULL);
      g_hash_table_destroy (prop->header);
    }

  if (prop->childs)
    uc_check_free_link_properties_list (prop->childs);

  g_free (prop);
}


/*
 * set a timeout header
 */
static GHashTable *
uc_check_url_get_header_timeout ()
{
  GHashTable *tmp = NULL;

  if (uc_check_timeout_flag_get ())
    return NULL;

  tmp = g_hash_table_new (g_str_hash, g_str_equal);
  g_hash_table_insert (tmp, g_strdup ("Status"),
		       g_strdup (STATUS_CODE_NO_DATA));
  g_hash_table_insert (tmp, g_strdup (CONTENT_TYPE),
		       g_strdup (CONTENT_TYPE_HTML));

  return tmp;
}


/*
 * set a item ignored header
 */
static GHashTable *
uc_check_url_get_header_ignored ()
{
  GHashTable *tmp = NULL;

  tmp = g_hash_table_new (g_str_hash, g_str_equal);
  g_hash_table_insert (tmp, g_strdup ("Status"),
		       g_strdup (STATUS_CODE_IGNORED_LINK));
  g_hash_table_insert (tmp, g_strdup (CONTENT_TYPE),
		       g_strdup (CONTENT_TYPE_HTML));

  return tmp;
}


/*
 * free all the allocated objects of the
 * url properties list
 */
void
uc_check_free_link_properties_list (GList * list)
{
  UCLinkProperties *prop = NULL;
  GList *item = NULL;

  if (!list)
    return;

  item = g_list_first (list);
  while (item)
    {
      prop = (UCLinkProperties *) item->data;

      g_free (prop->label);
      g_free (prop->url_original);
      g_free (prop->port);
      g_free (prop->url);
      g_free (prop->path);
      g_free (prop->file);
      g_free (prop->args);
      g_free (prop->h_name);
      g_free (prop->h_address);
      g_free (prop->protocol);

      if (prop->header != NULL)
	{
	  g_hash_table_foreach (prop->header,
				uc_check_free_link_properties_list_hash_table,
				NULL);
	  g_hash_table_destroy (prop->header);
	}

      if (prop->childs)
	uc_check_free_link_properties_list (prop->childs);

      g_free (prop);
      item = item->next;
    }

  g_list_free (list);

  list = NULL;
}


/*
 * send request for the page content
 */
static void
uc_check_gdk_input_get_content_send (gpointer buffer, gint fd,
				     GdkInputCondition in)
{
  gint nbr_send = -1;

  uc_check_alarm_add (CHECK_TIMEOUT, &uc_check_alarm_callback);
  nbr_send = send (fd, (gchar *) buffer, strlen ((gchar *) buffer), 0);
  uc_check_alarm_remove ();

  if (nbr_send <= 0)
    uc_check_timeout_flag = TRUE;

  uc_check_gdk_input_end = TRUE;
  uc_check_progress_bar_set_pulse ();
  uc_application_update_ui ();
}


/*
 * read the page content of the given descriptor
 */
static void
uc_check_gdk_input_get_content_read (gpointer buffer, gint fd,
				     GdkInputCondition in)
{
  UCGetContentReadData *read_data = (UCGetContentReadData *) buffer;
  gint nbr_read = -1;
  gchar buffer_recv[BUFFER_LEN + 1] = { 0 };

  uc_check_alarm_add (CHECK_TIMEOUT, &uc_check_alarm_callback);
  nbr_read = read (fd, buffer_recv, BUFFER_LEN);
  uc_check_alarm_remove ();

  if (nbr_read > 0)
    read_data->string = g_string_append (read_data->string, buffer_recv);
  else if (nbr_read <= 0)
    {
      if (nbr_read < 0)
	uc_check_timeout_flag = TRUE;

      uc_check_gdk_input_end = TRUE;
    }

  if (read_data->content_length)
    {
      if (read_data->string->len)
	uc_check_progress_bar_set_percent ((gfloat)
					   (((read_data->string->len * 100) /
					     (read_data->content_length +
					      read_data->header_length)) /
					    100));
    }
  else
    uc_check_progress_bar_set_pulse ();

  uc_application_update_ui ();
}


/*
 * send request for the page header
 */
static void
uc_check_gdk_input_get_header_send (gpointer buffer, gint fd,
				    GdkInputCondition in)
{
  gint nbr_send = -1;

  uc_check_alarm_add (CHECK_TIMEOUT, &uc_check_alarm_callback);
  nbr_send = send (fd, (gchar *) buffer, strlen ((gchar *) buffer), 0);
  uc_check_alarm_remove ();

  if (nbr_send <= 0)
    uc_check_timeout_flag = TRUE;

  uc_check_gdk_input_end = TRUE;
  uc_check_progress_bar_set_pulse ();
  uc_application_update_ui ();
}


/*
 * read the page header on the given descriptor
 */
static void
uc_check_gdk_input_get_header_read (gpointer data, gint fd,
				    GdkInputCondition in)
{
  gint nbr_read = -1;

  uc_check_alarm_add (CHECK_TIMEOUT, &uc_check_alarm_callback);
  nbr_read = read (fd, (gchar *) data, BUFFER_LEN);
  uc_check_alarm_remove ();

  if (nbr_read < 0)
    uc_check_timeout_flag = TRUE;

  uc_check_gdk_input_end = TRUE;
  uc_check_progress_bar_set_pulse ();
  uc_application_update_ui ();
}


/*
 * put all the header's informations in a
 * hashtable
 */
static GHashTable *
uc_check_url_get_header (UCLinkProperties * prop)
{
  GHashTable *tmp = NULL;
  gchar *host = prop->h_name;
  gchar *path = prop->path;
  gchar *file = prop->file;
  gchar *port = prop->port;
  gchar buffer[BUFFER_LEN + 1] = { 0 };
  gchar **lines = NULL;
  gchar buffer_recv[BUFFER_LEN + 1] = { 0 };
  gchar str[HEADER_STATUS_CODE_LEN + 1] = { 0 };
  guint i = 0;
  guint sock = 0;
  gint gifd = 0;

  if ((sock =
       uc_utils_socket_open (host, atoi (port), UC_SOCKET_OPEN_DEFAULT)) <= 0)
    {
      /* simulate a timeout for the current host */
      if (!uc_check_timeout_flag)
	uc_check_alarm_callback (0);

      return NULL;
    }

  snprintf (buffer, BUFFER_LEN, "HEAD %s://%s%s%s HTTP/1.0\r\n"
	    "Host: %s\r\n"
	    "User-agent: %s\r\n"
	    "Referer: %s\r\n"
	    "\r\n",
	    prop->protocol, host, path, file, host, USER_AGENT, REFERER);

  uc_check_display_state_message (CHECK_MESSAGE_LABEL_SECOND, path);
  uc_check_display_state_message (CHECK_MESSAGE_LABEL_THIRD,
				  _("Retrieving header..."));
  uc_application_update_ui ();

  /* process the sending in background */
  uc_check_gdk_input_end = FALSE;
  gifd =
    gdk_input_add (sock, GDK_INPUT_WRITE,
		   (GdkInputFunction) uc_check_gdk_input_get_header_send,
		   &buffer);

  /* wait for the end of the sending operation */
  while (UC_CHECK_IO_WHILE_CONDITION)
    uc_application_update_ui ();

  gdk_input_remove (gifd);

  /* quit if a timeout occured */
  if (uc_check_timeout_flag)
    {
      close (sock);
      return NULL;
    }

  /* process the reading in background */
  uc_check_gdk_input_end = FALSE;
  gifd =
    gdk_input_add (sock, GDK_INPUT_READ,
		   (GdkInputFunction) uc_check_gdk_input_get_header_read,
		   &buffer_recv);

  /* wait for the end of the reading operation */
  while (UC_CHECK_IO_WHILE_CONDITION)
    uc_application_update_ui ();

  gdk_input_remove (gifd);

  /* quit if a timeout occured */
  if (uc_check_timeout_flag)
    {
      close (sock);
      return NULL;
    }

  if (!strlen (buffer_recv))
    return (uc_check_ignore_item_value) ?
      uc_check_url_get_header_ignored () : uc_check_url_get_header_timeout ();

  g_memmove (&str, &buffer_recv[HEADER_STATUS_CODE_FIRST_POS],
	     HEADER_STATUS_CODE_LEN);

  tmp = g_hash_table_new (g_str_hash, g_str_equal);
  g_hash_table_insert (tmp, g_strdup ("Status"), g_strdup (str));

  prop->header_size = strlen (buffer_recv);
  i = 0;
  lines = g_strsplit (buffer_recv, "\r\n", 0);
  while (lines[i])
    {
      gchar **key_value = g_strsplit (lines[i], ":", 2);

      if (key_value[0] && key_value[1])
	g_hash_table_insert (tmp,
			     g_strstrip (g_strdup (key_value[0])),
			     g_strstrip (g_strdup (key_value[1])));
      g_strfreev (key_value);
      i++;
    }
  g_strfreev (lines);

  close (sock);

  return tmp;
}


/*
 * if the beginning of the given gchar buffer pointer
 * is in our "to check" types, it return the link type (HREF, IMG...)
 *
 * -> the second arg is modified by this function
 */
static UCTagLinkProperties *
uc_check_get_tag_link_properties (gchar * buffer)
{
  gchar local_url[BUFFER_URL_LEN + 1] = { 0 };
  UCTagLinkProperties *tag_link = NULL;

  /* return NULL if the given buffer is not big enough */
  if ((!buffer) || strlen (buffer) < SEARCH_STRING_MAX_LEN + 2)
    return NULL;

  tag_link = (UCTagLinkProperties *) g_malloc (sizeof (UCTagLinkProperties));
  tag_link->type = LINK_TYPE_NONE;

  /* HREF check */
  if (uc_utils_memcasecmp (buffer, SEARCH_STRING_DOCUMENTS))
    tag_link->type = LINK_TYPE_HREF;

  /* IMG check */
  else if (uc_utils_memcasecmp (buffer, SEARCH_STRING_IMAGES_TAG))
    tag_link->type = LINK_TYPE_IMAGE;

  if (tag_link->type != LINK_TYPE_NONE)
    {
      guint k = 0;
      guint i = 0;
      gchar *begin = NULL;
      gchar *end = NULL;

      /* retreive link value */
      if (tag_link->type == LINK_TYPE_HREF)
	{
	  i += SEARCH_STRING_DOCUMENTS_LEN;
	  while ((k < BUFFER_URL_LEN) && (buffer[i]) && (buffer[i] != '"'))
	    local_url[k++] = buffer[i++];

	  local_url[k] = '\0';
	  tag_link->value = g_strdup (local_url);
	}
      else
	{
	  gchar *tmp = NULL;

	  i += SEARCH_STRING_IMAGES_TAG_LEN;
	  begin =
	    uc_utils_search_string_next (&buffer[i],
					 SEARCH_STRING_IMAGES_SRC_TAG, '>');
	  end = begin;
	  while (end && (*end != '<') && (*end != ' '))
	    end++;

	  tmp = uc_utils_strdup_delim (begin, end);
	  tag_link->value = uc_utils_clean_tag_link_value (tmp);
	  g_free (tmp);
	  i += SEARCH_STRING_IMAGES_SRC_TAG_LEN;
	}

      /* retreive link label */
      if ((buffer[i]) && (tag_link->type != LINK_TYPE_IMAGE))
	{
	  begin = &buffer[i];
	  do
	    begin = strpbrk (begin, ">");
	  while (begin && (*(++begin) == '<'));
	  end = strpbrk (begin, "<");

	  if (end)
	    {
	      gchar *tmp = NULL;

	      tmp = uc_utils_strdup_delim (begin, end);
	      tag_link->label = uc_check_format_list_label (tmp);
	      g_free (tmp);
	    }
	  else
	    tag_link->label = g_strdup (_("<image link>"));
	}
      else
	{
	  gchar *tmp = g_strdup (_("<no label>"));

	  tag_link->label = uc_check_format_list_label (tmp);

	  g_free (tmp);
	}

      if (!strlen (tag_link->value))
	{
	  uc_check_free_tag_link_properties_node ((gpointer) tag_link);
	  tag_link = NULL;
	}
    }
  else
    g_free (tag_link), tag_link = NULL;

  return tag_link;
}


/*
 * free a UCTagLinkProperties node
 */
static void
uc_check_free_tag_link_properties_node (UCTagLinkProperties * prop)
{
  g_free (prop->label);
  g_free (prop->value);

  g_free (prop);
}


/*
 * get all the links of a given buffer
 */
static GList *
uc_check_url_get_links_real (UCLinkProperties * prop,
			     const gchar * host, const guint depth)
{
  GList *list = NULL;
  UCLinkProperties *lp = NULL;
  gchar *begin = NULL;
  gchar *str = NULL;
  gchar *str_hash = NULL;
  gchar *content_type = NULL;
  gchar *current_host = NULL;
  gchar *current_port = NULL;
  gchar *current_path = NULL;
  gchar *current_file = NULL;
  gchar *current_args = NULL;
  gboolean good_page = TRUE;
  guint check_level_tmp = 0;
  UCTagLinkProperties *tag_link = NULL;

  uc_url_parse ("", "", prop->url, &current_host, &current_port,
		&current_path, &current_file, &current_args);

  uc_check_ignore_item_value = FALSE;

  if ((str =
       uc_check_url_get_content (prop, current_path, current_file,
				 current_args, current_host)))
    {
      uc_cache_append_source (prop->uid, str);

      begin = str;
      while (UC_CHECK_PARSER_WHILE_CONDITION)
	{
	  uc_application_update_ui ();
	  begin = strpbrk (begin, "<");

	  if (!begin)
	    break;

	  /* we don't want to check html comments */
	  if (*(begin + 1) == '!')
	    begin = strpbrk (begin, ">");

	  if ((tag_link = uc_check_get_tag_link_properties (begin)))
	    {
	      /* if we are on a link, check it */
	      gchar *protocol = NULL;

	      begin += strlen (tag_link->value);

	      if ((!strstr (tag_link->value, "ad.doubleclick"))
		  && (!uc_check_url_in_checked_links (tag_link->value)))
		{
		  /* if the protocol is ok for checking (HTTP etc.) */
		  protocol = uc_url_get_protocol (tag_link->value);

		  if (uc_check_protocol_accepted (protocol))
		    {
		      uc_check_wait ();

		      lp = NULL;

		      if ((!strstr (tag_link->value, "mailto:")) &&
			  (!strstr (tag_link->value, "javascript:")))
			lp =
			  uc_check_link_get_properties (current_host,
							current_path,
							tag_link->value);

		      if (lp)
			{
			  if (lp->header)
			    {
			      content_type =
				g_hash_table_lookup (lp->header,
						     CONTENT_TYPE);

			      /* must we skip this content-type? */
			      if (!uc_check_content_type_accepted
				  (content_type))
				uc_check_free_properties_node ((gpointer) lp);

			      /* add this page if it doesn't yet exist in
			       * the list */
			      else if (check_level_tmp++ <
				       projectState.check_level)
				{
				  lp->uid = uc_check_node_uid++;
				  lp->link_type = tag_link->type;
				  lp->link_icon =
				    uc_check_get_link_icon (tag_link->type);
				  lp->depth_level = depth;
				  lp->label =
				    uc_check_format_list_label (tag_link->
								label);
				  lp->is_parsable =
				    (strstr (content_type, "text/html") !=
				     NULL);

				  /* all status code not beginning with 2
				   * or 3 are considered bad codes */
				  str_hash =
				    g_hash_table_lookup (lp->header,
							 "Status");
				  good_page =
				    (!(str_hash[0] != '2'
				       && str_hash[0] != '3'));

				  list = g_list_append (list, (gpointer) lp);
				  uc_check_tmp_list =
				    g_list_prepend (uc_check_tmp_list,
						    g_strdup (lp->
							      url_original));

				  /* current informations for timeouts
				   * callback */
				  uc_check_currentitem_set_list ((gpointer)
								 list);

				  if (str_hash[0] == '9')
				    uc_report_set_timedoutlinks (0);
				  else
				    {
				      good_page =
					(!(str_hash[0] != '2'
					   && str_hash[0] != '3'));
				      (!good_page) ?
					uc_report_set_badlinks (0) :
					uc_report_set_goodlinks (0);
				    }

				  uc_report_display_update ();
				}
			      else
				uc_check_free_properties_node ((gpointer) lp);
			    }
			}
		    }
		  g_free (protocol);

		  uc_check_free_tag_link_properties_node ((gpointer)
							  tag_link);
		}
	      else if (begin)
		++begin;
	    }
	  else if (begin)
	    ++begin;
	}
    }

  g_free (str);
  g_free (current_host);
  g_free (current_port);
  g_free (current_path);
  g_free (current_file);
  g_free (current_args);

  return list;
}


/*
 * simply return content of a given url
 */
static gchar *
uc_check_url_get_content_bis (UCLinkProperties * prop)
{
  gchar *current_host = NULL;
  gchar *current_port = NULL;
  gchar *current_path = NULL;
  gchar *current_file = NULL;
  gchar *current_args = NULL;
  gchar *str = NULL;

  uc_url_parse ("", "", prop->url, &current_host, &current_port,
		&current_path, &current_file, &current_args);
  str = uc_check_url_get_content (prop, current_path, current_file,
				  current_args, current_host);

  g_free (current_host);
  g_free (current_port);
  g_free (current_path);
  g_free (current_file);
  g_free (current_args);

  return str;
}


/*
 * put in a list all the urls of
 * a given url
 */
static GList *
uc_check_url_get_links (const gchar * basehost,
			UCLinkProperties * prop,
			const guint depth, GList * list_all)
{
  gchar *host = NULL;
  GList *urls = NULL;
  GList *item = NULL;
  GList *tmp_list = NULL;

  if (!prop)
    return NULL;

  host = uc_url_get_hostname ("", prop->url);
  if (uc_check_cancel_value || (!prop->url) || g_strcasecmp (basehost, host))
    {
      g_free (host);
      return NULL;
    }

  if (depth == projectState.depth_level)
    {
      gchar *str = NULL;

      if ((str = uc_check_url_get_content_bis (prop)))
	uc_cache_append_source (prop->uid, str);

      g_free (host);
      g_free (str);

      return NULL;
    }

  uc_check_display_state_message (CHECK_MESSAGE_LABEL_FIRST, prop->url);
  uc_application_update_ui ();

  urls = uc_check_url_get_links_real (prop, host, depth);
  item = g_list_first (urls);
  while (item)
    {
      UCLinkProperties *lp = (UCLinkProperties *) item->data;

      if (lp->is_parsable)
	lp->childs =
	  uc_check_url_get_links (basehost, lp, depth + 1, list_all);
      tmp_list = g_list_append (tmp_list, (gpointer) lp);

      item = g_list_next (item);
    }

  g_free (host);
  g_list_free (urls);

  return tmp_list;
}


/*
 * TRUE if the given url already in
 * the checked urls list
 */
static gboolean
uc_check_url_in_checked_links (gchar * url)
{
  GList *item = g_list_first (uc_check_tmp_list);

  while (item)
    {
      if (!g_strcasecmp ((gchar *) item->data, url))
	return TRUE;
      item = g_list_next (item);
    }

  return FALSE;
}


/*
 * free all allocated url strings
 */
void
uc_check_free_tmp_list (void)
{
  if (!uc_check_tmp_list)
    return;

  g_list_foreach (uc_check_tmp_list, (GFunc) g_free, NULL);
  g_list_free (uc_check_tmp_list);

  uc_check_tmp_list = NULL;
}


/*
 * funtion passed to g_hash_table_foreach() to
 * free all the allocated object
 */
static void
uc_check_free_link_properties_list_hash_table (gpointer key,
					       gpointer value,
					       gpointer user_data)
{
  g_free (key);
  g_free (value);
}


/*
 * set a SIGALRM
 */
void
uc_check_alarm_add (const guint seconds, gpointer callback)
{
  struct sigaction action;

  uc_check_timeout_flag_set (FALSE);
  uc_utils_block_signals ();
  sigemptyset (&(action.sa_mask));
  action.sa_flags = 0;
  action.sa_handler = callback;

  sigaction (SIGALRM, &action, NULL);
  alarm (seconds);
}


/*
 * remove a SIGALRM
 */
void
uc_check_alarm_remove (void)
{
  uc_utils_block_signals ();
  alarm (0);
}


/*
 * dislay items in a frame
 */
void
uc_check_buttons_display (const GtkWidget * frame, gpointer array,
			  const guint items_size,
			  gpointer selected_array,
			  const guint selected_items_size)
{
  GtkWidget *checkButton = NULL;
  GtkWidget *table = NULL;
  gchar **items = (gchar **) array;
  gchar **selected_items = (gchar **) selected_array;
  guint i = 0;
  guint col = 0;
  guint row = 0;
  gdouble maxRows = 0;
  gdouble maxCols = DIALOG_SETTINGS_DOCUMENT_TYPES_MAX_COLS;

  maxRows = ceil (items_size / maxCols);

  table = gtk_table_new (maxCols, maxRows, FALSE);
  gtk_widget_show (table);
  gtk_container_add (GTK_CONTAINER (frame), table);
  gtk_container_set_border_width (GTK_CONTAINER (table), 5);
  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
  gtk_table_set_col_spacings (GTK_TABLE (table), 2);

  for (row = 0, i = 0; row < maxRows; row++, col = 0)
    while ((col < maxCols) && (i < items_size))
      {
	checkButton = gtk_check_button_new_with_mnemonic (items[i]);

	if (uc_utils_array_is_in
	    (items[i], selected_items, selected_items_size))
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkButton),
					TRUE);
	gtk_widget_show (checkButton);
	gtk_table_attach (GTK_TABLE (table), checkButton, col, col + 1,
			  row, row + 1, (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	col++, i++;
      }
}


/*
 * -- FIXME --
 * return a GList of pointers to the check buttons
 * of the notebooks
 */
static GList *
uc_check_buttons_get_list (const GtkWidget * widget)
{
  GList *parents = NULL;
  GList *childs = NULL;
  GList *list = NULL;

  parents = gtk_container_get_children (GTK_CONTAINER (widget));
  while (parents)
    {
      if (GTK_IS_TABLE (parents->data))
	{
	  childs = gtk_container_get_children (GTK_CONTAINER (parents->data));
	  while (childs)
	    {
	      list = g_list_prepend (list, childs->data);
	      childs = childs->next;
	    }
	  g_list_free (childs);
	}
      parents = parents->next;
    }

  g_list_free (parents);

  return list;
}


/*
 * saving states of a notebook check
 * button
 */
void
uc_check_buttons_save_setting (GtkWidget * widget, gchar * path)
{
  GList *items = NULL;
  gchar **radiosArray = NULL;
  guint size = 0;
  GString *radiosStr = g_string_new ("");

  items = uc_check_buttons_get_list (widget);
  while (items)
    {
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (items->data)))
	{
	  g_string_append_printf (radiosStr, ",%s",
				  gtk_button_get_label (GTK_BUTTON
							(items->data)));
	  size++;
	}
      items = items->next;
    }
  g_list_free (items);

  if (size)
    g_string_erase (radiosStr, 0, 1);

  radiosArray = g_strsplit (radiosStr->str, ",", CHECK_BUTTON_MAX_DISPLAY);

  UC_CONFIG_SET_VECTOR (path, (const gchar **) radiosArray, size);

  g_strfreev (radiosArray);
}


/*
 * set the cancel check flag.
 * this flag is tested during the check to know
 * if the check must aborting or continuing
 */
void
uc_check_cancel (void)
{
  uc_check_cancel_value = TRUE;
}


void
uc_check_ignore_item (void)
{
  uc_check_ignore_item_value = TRUE;
}


/*
 * return a pointer on the url list
 */
GList *
uc_check_get_properties_list (void)
{
  return uc_link_properties_list;
}


/*
 * return the id of the current selected row in the url
 * treeview
 */
gint
uc_check_treeview_get_selected_row_id (void)
{
  GtkTreeIter iter;
  GtkTreeModel *model = NULL;
  GtkTreeSelection *select =
    gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
  gint ret = -1;

  if (gtk_tree_selection_get_selected (select, &model, &iter))
    gtk_tree_model_get (model, &iter, 0, &ret, -1);

  return ret;
}


/*
 * search the header of a given url
 * in the urls list to display it
 */
gboolean
uc_check_display_informations (GtkTreeView * treeview,
			       const UCDisplayFormat display_format)
{
  GtkWidget *display_button = NULL;
  gint id = 0;
  UCLinkProperties *prop = NULL;
  UCStatusCode *sc = NULL;
  gchar *status_code = NULL;
  gchar *tmp_str = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkWidget *text_informations = NULL;
  GtkWidget *scrolled_window_informations = NULL;
  gboolean simple_display =
    ((projectState.display_format == SIMPLE_PRINTING)
     && (display_format == DEFAULT_PRINTING))
    || (display_format == SIMPLE_PRINTING);

  if ((id = uc_check_treeview_get_selected_row_id ()) < 0)
    {
      uc_application_dialog_show (_("Please, select a item first."),
				  GTK_MESSAGE_WARNING);

      return TRUE;
    }

  prop = uc_check_search_link_properties_list (uc_link_properties_list, id);

  UC_GET_CUSTOM_WIDGET (glade_main_window, main_window, "display_button",
			display_button);
  g_signal_handlers_destroy (G_OBJECT (display_button));
  UC_GET_CUSTOM_WIDGET (glade_main_window, main_window,
			"scrolled_window_informations",
			scrolled_window_informations);
  UC_GET_CUSTOM_WIDGET (glade_main_window, main_window, "text_informations",
			text_informations);

  status_code = g_hash_table_lookup (prop->header, "Status");
  if (!(sc = uc_application_get_status_code_properties (status_code)))
    sc = uc_application_get_status_code_properties (STATUS_CODE_UNKNOWN);

  tmp_str = g_strdup_printf ("<b><i>%s</i></b>", sc->message);
  gtk_label_set_markup (GTK_LABEL
			(glade_xml_get_widget
			 (glade_main_window, "label_informations_message")),
			tmp_str);
  g_free (tmp_str);

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_informations));

  /* "edit" menu options enabled for the selected row if we are not on a
   * timeout or on a bad link */
  if (status_code[0] != '9' && status_code[0] != '4')
    UC_WIDGET_SET_SENSITIVE (glade_main_window, "edit", TRUE);
  else
    UC_WIDGET_SET_SENSITIVE (glade_main_window, "edit", FALSE);

  if (simple_display)
    {
      tmp_str = g_strdup (sc->description);

      gtk_button_set_label (GTK_BUTTON (display_button),
			    _("Detailed printing"));
      g_signal_connect (G_OBJECT (display_button), "clicked",
			G_CALLBACK (on_btDetailedDisplayFormat_clicked),
			(gpointer) treeview);
    }
  else
    {
      if ((status_code[0] != '9')
	  && (strcmp (status_code, STATUS_CODE_TIMEOUT)))
	{
	  informations_buffer = NULL;
	  g_hash_table_foreach (prop->header,
				uc_check_display_informations_set_label,
				NULL);
	  tmp_str = informations_buffer;
	}
      else
	tmp_str =
	  g_strdup (_("No detailed information available for this link."));

      gtk_button_set_label (GTK_BUTTON (display_button), _("Basic printing"));
      g_signal_connect (G_OBJECT (display_button), "clicked",
			G_CALLBACK (on_btBasicDisplayFormat_clicked),
			(gpointer) treeview);
    }

  gtk_text_buffer_set_text (buffer, tmp_str, -1);

  g_free (tmp_str);

  gtk_widget_show (display_button);

  return TRUE;
}


/*
 * display the header informations for a
 * given url
 */
static void
uc_check_display_informations_set_label (gpointer key, gpointer value,
					 gpointer data)
{
  gchar *tmp = g_strdup_printf ("%s%s : %s\n",
				(informations_buffer) ? informations_buffer
				: "",
				(gchar *) key, (gchar *) value);

  g_free (informations_buffer);
  informations_buffer = tmp;
}


/*
 * set the progress bar evolution
 */
static void
uc_check_progress_bar_set_percent (gfloat percent)
{
  GtkWidget *progress_bar = NULL;

  UC_GET_CUSTOM_WIDGET (glade_progress_dialog, progress_dialog,
			"progress_bar", progress_bar);
  gtk_progress_set_percentage (GTK_PROGRESS (progress_bar), percent);
}


/*
 * set the progress bar evolution when we don't know
 * the percents
 */
static void
uc_check_progress_bar_set_pulse (void)
{
  GtkWidget *progress_bar = NULL;

  UC_GET_CUSTOM_WIDGET (glade_progress_dialog, progress_dialog,
			"progress_bar", progress_bar);
  gtk_progress_bar_pulse (GTK_PROGRESS_BAR (progress_bar));
}


/*
 * display messages in the progess dialog during
 * the check process
 */
static void
uc_check_display_state_message (const gint label_pos, const gchar * message)
{
  GtkWidget *label = NULL;
  gchar *tmp = NULL;

  if ((!message) || (!progress_dialog) || (!GTK_IS_WIDGET (progress_dialog)))
    return;

  if (label_pos != CHECK_MESSAGE_LABEL_THIRD)
    tmp = uc_check_format_progress_dialog_label (message);
  else
    tmp = g_strdup (message);

  switch (label_pos)
    {
    case CHECK_MESSAGE_LABEL_FIRST:
      label = glade_xml_get_widget (glade_progress_dialog, "firstLabel");
      break;

    case CHECK_MESSAGE_LABEL_SECOND:
      label = glade_xml_get_widget (glade_progress_dialog, "secondLabel");
      break;

    case CHECK_MESSAGE_LABEL_THIRD:
      {
	label = glade_xml_get_widget (glade_progress_dialog, "thirdLabel");
	break;
      }

    default:
      g_free (tmp);
      return;
    }

  gtk_label_set_text (GTK_LABEL (label), tmp);
  g_free (tmp);
}


/*
 * reset all items in the right frames
 */
static void
uc_check_right_frames_reset (void)
{
  GtkWidget *widget = NULL;

  if ((widget =
       UC_GET_CUSTOM_WIDGET_REAL (main_window, "label_informations_message")))
    gtk_label_set_text (GTK_LABEL (widget), "");

  if ((widget = UC_GET_CUSTOM_WIDGET_REAL (main_window, "text_informations")))
    gtk_text_buffer_set_text (gtk_text_view_get_buffer
			      (GTK_TEXT_VIEW (widget)), "", -1);

  if ((widget = UC_GET_CUSTOM_WIDGET_REAL (main_window, "display_button")))
    gtk_widget_hide (widget);

  uc_check_report_reset ();
}


/*
 * this function must be called before
 * any check, to initialize some elements
 */
void
uc_check_reset (void)
{
  uc_application_remove_paths ();
  uc_check_free_link_properties_list (uc_link_properties_list);
  uc_check_free_tmp_list ();
  uc_check_right_frames_reset ();
  uc_check_cancel_value = FALSE;
  uc_check_ignore_item_value = FALSE;
  uc_check_node_uid = 1;
}


/*
 * display recursively urls in the treeview
 */
void
uc_check_display_list (GList * list, GtkTreeIter iter[], guint i_iter)
{
  UCLinkProperties *prop = NULL;
  UCStatusCode *sc = NULL;
  gchar *status_code = NULL;
  GdkColor bgcolor;
  GList *item = NULL;

  item = g_list_first (list);
  while (item)
    {
      prop = (UCLinkProperties *) item->data;

      status_code = g_hash_table_lookup (prop->header, "Status");
      if (!prop->url)
	prop->url = g_strdup (prop->h_name);

      if ((sc = uc_application_get_status_code_properties (status_code)))
	bgcolor = sc->background;

      if (!prop->link_icon)
	prop->link_icon =
	  gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/uc_treeview_unknown.png",
				    NULL);

      if (!i_iter)
	{
	  gtk_tree_store_append (treestore, &iter[i_iter], NULL);
	  gtk_tree_store_set (treestore, &iter[0],
			      UID_COLUMN, prop->uid,
			      LINK_ICON_COLUMN,
			      prop->link_icon,
			      LABEL_COLUMN, prop->label, URL_COLUMN,
			      prop->url, BG_COLOR_COLUMN, &bgcolor, -1);
	}
      else
	{
	  gtk_tree_store_append (treestore, &iter[i_iter], &iter[i_iter - 1]);
	  gtk_tree_store_set (treestore, &iter[i_iter], UID_COLUMN,
			      prop->uid, LINK_ICON_COLUMN, prop->link_icon,
			      LABEL_COLUMN, prop->label, URL_COLUMN,
			      prop->url, BG_COLOR_COLUMN, &bgcolor, -1);
	}

      if (prop->childs)
	uc_check_display_list (prop->childs, iter, i_iter + 1);

      item = item->next;
    }
}


/*
 * display url list with a mask
 */
void
uc_check_display_list_with_only (GList * list, GtkTreeIter iter,
				 const guint view_type)
{
  GList *item = NULL;
  UCLinkProperties *prop = NULL;
  gchar *status_code = NULL;
  UCStatusCode *sc = NULL;

  GdkColor bgcolor;
  gboolean display = FALSE;

  item = g_list_first (list);
  while (item)
    {
      prop = (UCLinkProperties *) item->data;
      status_code = g_hash_table_lookup (prop->header, "Status");

      switch (view_type)
	{
	case UC_CHECK_DISPLAY_BADLINKS:
	  display =
	    ((status_code[0] != '2') && (status_code[0] != '9') &&
	     (strcmp (status_code, STATUS_CODE_REDIRECTED)) &&
	     (strcmp (status_code, STATUS_CODE_TIMEOUT)));
	  break;

	case UC_CHECK_DISPLAY_GOODLINKS:
	  display = ((status_code[0] == '2') ||
		     (!strcmp (status_code, STATUS_CODE_REDIRECTED)));
	  break;

	case UC_CHECK_DISPLAY_TIMEOUTS:
	  display =
	    ((status_code[0] == '9') ||
	     (!strcmp (status_code, STATUS_CODE_TIMEOUT)));
	  break;
	}

      if (display)
	{
	  if (!prop->url)
	    prop->url = g_strdup (prop->h_name);
	  if ((sc = uc_application_get_status_code_properties (status_code)))
	    bgcolor = sc->background;
	  gtk_tree_store_append (treestore, &iter, NULL);

	  if (!prop->link_icon)
	    prop->link_icon =
	      gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR
					"/uc_treeview_unknown.png", NULL);

	  gtk_tree_store_set (treestore, &iter,
			      UID_COLUMN, prop->uid,
			      LINK_ICON_COLUMN,
			      prop->link_icon,
			      LABEL_COLUMN, prop->label, URL_COLUMN,
			      prop->url, BG_COLOR_COLUMN, &bgcolor, -1);
	}

      if (prop->childs)
	uc_check_display_list_with_only (prop->childs, iter, view_type);

      item = item->next;
    }

}


/*
 * begin the page urls check
 */
void
uc_check_begin (void)
{
  time_t t;
  UCLinkProperties *prop = NULL;
  gchar *page = NULL;
  gchar *host = NULL;
  GtkCellRenderer *renderer = NULL;
  GtkTreeViewColumn *column = NULL;
  guint timer_id = 0;
  gchar *tmp = NULL;
  GtkTreeIter iter[10];
  guint i_iter = 0;

  uc_application_draw_main_frames ();
  uc_application_update_ui ();

  treeview =
    (GtkTreeView *) UC_GET_CUSTOM_WIDGET_REAL (main_window, "urls_list");

  uc_application_progress_dialog_show ();
  uc_application_update_ui ();

  page = projectState.url;
  host = uc_url_get_hostname (NULL, page);
  tmp = g_strdup_printf (_("Checking %s..."), page);
  uc_check_display_state_message (CHECK_MESSAGE_LABEL_FIRST, tmp);
  uc_application_update_ui ();
  g_free (tmp);

  uc_check_reset ();

  time (&t);
  timer_id = gtk_timeout_add (1000, &uc_report_timer_callback, (gpointer) t);

  prop = uc_check_link_get_properties (host, "", page);
  uc_link_properties_list =
    uc_check_url_get_links (host, prop, 0, uc_link_properties_list);
  uc_check_free_properties_node (prop);
  g_free (host);

  gtk_timeout_remove (timer_id);
  i_iter = 0;
  treestore = gtk_tree_store_new (N_COLUMNS,
				  G_TYPE_INT,
				  GDK_TYPE_PIXBUF,
				  G_TYPE_STRING,
				  G_TYPE_STRING, GDK_TYPE_COLOR);

  uc_check_display_list (uc_link_properties_list, iter, i_iter);

  gtk_tree_view_set_model (GTK_TREE_VIEW (treeview),
			   GTK_TREE_MODEL (treestore));

  /* ID column */
  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
					       -1,
					       "Id",
					       renderer,
					       "text", UID_COLUMN, NULL);
  column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), UID_COLUMN);
  gtk_tree_view_column_set_sort_column_id (column, UID_COLUMN);
  gtk_tree_view_column_set_resizable (column, TRUE);

  /* LINK TYPE column */
  renderer = gtk_cell_renderer_pixbuf_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
					       -1,
					       "Type",
					       renderer,
					       "pixbuf", LINK_ICON_COLUMN,
					       NULL);
  column =
    gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), LINK_ICON_COLUMN);
  gtk_tree_view_column_set_sort_column_id (column, LINK_ICON_COLUMN);
  gtk_tree_view_column_set_resizable (column, TRUE);


  /* LABEL column */
  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
					       -1,
					       "Label",
					       renderer,
					       "text", LABEL_COLUMN, NULL);
  column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), LABEL_COLUMN);
  gtk_tree_view_column_set_sort_column_id (column, LABEL_COLUMN);
  gtk_tree_view_column_set_resizable (column, TRUE);

  /* URL column */
  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
					       -1,
					       "URL",
					       renderer,
					       "text", URL_COLUMN,
					       "background-gdk",
					       BG_COLOR_COLUMN, NULL);
  column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), URL_COLUMN);
  gtk_tree_view_column_set_resizable (column, TRUE);
  gtk_widget_destroy (progress_dialog);

  /* we have a list, the we have display options */
  uc_check_display_items_active_all ();

  /* treeview is collapsed by default and no row is selected, so we deactivate 
     the "Collapse all the list" and "edit" menu options */
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_all", FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_collapse_all", FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "edit", FALSE);
}


/*
 * activate menu display options
 */
void
uc_check_display_items_active_all (void)
{
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display", TRUE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_all", TRUE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_badlinks", TRUE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_goodlinks", TRUE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_timeouts", TRUE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_collapse_all", TRUE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_expand_all", TRUE);
}


/*
 * the callback to pass to uc_check_alarm_add()
 * function
 */
void
uc_check_alarm_callback (int dum)
{
  gchar *host = currentItem.host;
  gchar *url = currentItem.url;
  gchar *tmp = (url) ? url : host;

  uc_check_timeout_flag = TRUE;

  if (!uc_check_url_in_checked_links (tmp))
    {
      uc_check_insert_badlink ();
      uc_report_set_timedoutlinks (0);
    }

  uc_report_display_update ();
}


/*
 * insert a bad item in the url list (typically
 * timeout or bad host)
 */
static void
uc_check_insert_badlink (void)
{
  UCLinkProperties *prop = NULL;
  GList *list = currentItem.list_ptr;
  gchar *url = currentItem.url;
  gchar *host = currentItem.host;
  gchar *tmp = NULL;

  prop = (UCLinkProperties *) g_malloc (sizeof (UCLinkProperties));
  prop->uid = uc_check_node_uid++;
  prop->link_type = LINK_TYPE_NONE;
  prop->link_icon = uc_check_get_link_icon (LINK_TYPE_NONE);
  g_strdup (_("<no label>"));
  prop->label = uc_check_format_list_label (tmp);
  g_free (tmp);
  prop->header = g_hash_table_new (g_str_hash, g_str_equal);
  prop->header_size = 0;
  prop->url = (url) ? g_strdup (url) : NULL;
  prop->protocol =
    (url) ? uc_url_get_protocol (url) : g_strdup (PROTOCOL_HTTP);
  prop->port = g_strdup (URL_DEFAULT_PORT);
  prop->h_name = (host) ? g_strdup (host) : NULL;
  prop->childs = NULL;
  prop->url_original = g_strdup ("");
  prop->path = g_strdup ("");
  prop->file = g_strdup ("");
  prop->args = g_strdup ("");
  prop->h_address = g_strdup ("");;

  if (prop->url)
    g_hash_table_insert (prop->header, g_strdup ("Status"),
			 g_strdup (STATUS_CODE_TIMEOUT));
  else
    {
      prop->url = g_strdup (prop->h_name);
      g_hash_table_insert (prop->header, g_strdup ("Status"),
			   g_strdup (STATUS_CODE_HOST_IS_UNREACHABLE));
    }

  list = g_list_append (list, (gpointer) prop);
  uc_check_tmp_list =
    g_list_prepend (uc_check_tmp_list, g_strdup (prop->url));
}


/*
 * return TRUE if the given content-type is authorized
 * to be checked
 */
static gboolean
uc_check_content_type_accepted (const gchar * content_type)
{
  gchar **documents;
  gchar **images;
  guint documents_size = 0;
  guint images_size = 0;
  guint i = 0;

  if (!content_type)
    return FALSE;

  UC_CONFIG_GET_VECTOR (UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_DOCUMENTS,
			&documents, &documents_size);
  UC_CONFIG_GET_VECTOR (UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_IMAGES,
			&images, &images_size);

  if (!strlen (documents[0]))
    documents_size = 0;
  if (!strlen (images[0]))
    images_size = 0;

  for (i = 0; i < documents_size; i++)
    if (strstr (content_type, documents[i]))
      {
	g_strfreev (images);
	g_strfreev (documents);
	return FALSE;
      }
  g_strfreev (documents);

  for (i = 0; i < images_size; i++)
    if (strstr (content_type, images[i]))
      {
	g_strfreev (images);
	return FALSE;
      }
  g_strfreev (images);

  return TRUE;
}


/*
 * return TRUE if the given protocol is authorized
 * to be checked.
 */
static gboolean
uc_check_protocol_accepted (gchar * protocol)
{
  gchar **accepted = UC_CHECK_PROTOCOLS_GET_ACCEPTED;
  guint i = 0;

  g_strup (protocol);
  for (i = 0; i < UC_CHECK_PROTOCOLS_ACCEPTED_LEN; i++)
    if (!strcmp (accepted[i], protocol))
      {
	g_strfreev (accepted);
	return TRUE;
      }

  g_strfreev (accepted);
  return FALSE;
}


/*
 * wait
 */
static void
uc_check_wait (void)
{
  sleep (CHECK_INTERVAL_WAIT);
}


/*
 * set the host in the current item struct
 * for url list timeout management
 */
void
uc_check_currentitem_set_host (gchar * host)
{
  currentItem.host = host;
  currentItem.url = NULL;
}


/*
 * set the list in the current item struct
 * for url list timeout management
 */
static void
uc_check_currentitem_set_list (gpointer * list)
{
  currentItem.list_ptr = (GList *) list;
}


/*
 * fill the UCLinkProperties struct to associate with
 * the url
 */
static UCLinkProperties *
uc_check_link_get_properties (const gchar *
			      current_host,
			      const gchar * current_path, gchar * url)
{
  UCLinkProperties *prop = NULL;

  uc_check_display_state_message (CHECK_MESSAGE_LABEL_SECOND, url);
  uc_check_display_state_message (CHECK_MESSAGE_LABEL_THIRD,
				  _("Getting page properties..."));
  uc_application_update_ui ();

  prop = (UCLinkProperties *) g_malloc (sizeof (UCLinkProperties));
  prop->uid = 0;
  prop->link_type = LINK_TYPE_NONE;
  prop->link_icon = NULL;
  prop->url_original = g_strdup (url);
  prop->protocol = uc_url_get_protocol (url);
  prop->childs = NULL;
  prop->depth_level = 0;
  prop->is_parsable = FALSE;
  prop->label = NULL;
  uc_url_parse (current_host, current_path, url, &prop->h_name,
		&prop->port, &prop->path, &prop->file, &prop->args);
  prop->h_address = uc_url_get_ip (prop->h_name);
  prop->header_size = 0;
  prop->header = uc_check_url_get_header ((gpointer) prop);

  if (!prop->header)
    {
      g_free (prop->protocol);
      g_free (prop->url_original);
      g_free (prop->h_name);
      g_free (prop->port);
      g_free (prop->path);
      g_free (prop->file);
      g_free (prop->args);
      g_free (prop->h_address);

      g_free (prop);

      return NULL;
    }

  if (!strcmp
      (g_hash_table_lookup (prop->header, "Status"),
       STATUS_CODE_MOVED_PERMANENTLY))
    g_hash_table_insert (prop->header, g_strdup ("Status"),
			 g_strdup (STATUS_CODE_PAGE_OK));

  prop->url = g_strdup_printf ("%s://%s:%s%s%s%s",
			       g_strdown (prop->protocol),
			       prop->h_name,
			       prop->port,
			       prop->path, prop->file, prop->args);

  return prop;
}


/*
 * recursively search a item in the
 * url properties list
 */
static UCLinkProperties *
uc_check_search_link_properties_list (GList * list, const guint id)
{
  GList *item = NULL;
  UCLinkProperties *dum = NULL;
  UCLinkProperties *lp = NULL;

  item = g_list_first (list);
  while (item)
    {
      lp = (UCLinkProperties *) item->data;

      if (lp->uid == id)
	return lp;

      if (lp->childs)
	if ((dum = uc_check_search_link_properties_list (lp->childs, id)))
	  return dum;

      item = item->next;
    }

  return NULL;
}
