/*
 * Copyright (C) 2002, 2003 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef H_UC_GENERAL
#define H_UC_GENERAL

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include <math.h>
#include <glade/glade.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>

#define PROPERTIES_FILENAME "/gurlchecker-properties"
#define GLADE_XML_FILENAME PACKAGE_DATA_DIR "/" PACKAGE "/gui/gurlchecker.glade"
#define UC_PIXMAPS_DIR PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps/"

#define DIALOG_SETTINGS_DOCUMENT_TYPES_MAX_COLS 6

#define CHECK_BUTTON_MAX_DISPLAY 100

#define CHECK_MESSAGE_LABEL_FIRST 0
#define CHECK_MESSAGE_LABEL_SECOND 1
#define CHECK_MESSAGE_LABEL_THIRD 2

#define UC_CHECK_DISPLAY_GOODLINKS 0
#define UC_CHECK_DISPLAY_BADLINKS 1
#define UC_CHECK_DISPLAY_TIMEOUTS 10

#define STATUS_CODE_PAGE_OK "200"
#define STATUS_CODE_NO_DATA "204"
#define STATUS_CODE_MOVED_PERMANENTLY "301"
#define STATUS_CODE_REDIRECTED "302"
#define STATUS_CODE_TIMEOUT "408"
#define STATUS_CODE_HOST_IS_UNREACHABLE "999"
#define STATUS_CODE_IGNORED_LINK "997"
#define STATUS_CODE_UNKNOWN "998"
#define STATUS_CODE_PROPERTIES_ARRAY_LEN 7

#define STATUS_CODE_PROPERTIES_CODE 0
#define STATUS_CODE_PROPERTIES_MESSAGE 1
#define STATUS_CODE_PROPERTIES_DESCRIPTION 2
#define STATUS_CODE_PROPERTIES_BACKGROUND 3

#define IGNORED_LINK_BGCOLOR "pink"
#define BAD_LINK_BGCOLOR "red"
#define GOOD_LINK_BGCOLOR "green"
#define TIMEOUT_LINK_BGCOLOR "purple"
#define HOST_UNREACHABLE_LINK_BGCOLOR "yellow"
#define NO_DATA_LINK_BGCOLOR "orange"

#define UC_CHECK_PROTOCOLS_ACCEPTED "HTTP,FTP"
#define UC_CHECK_PROTOCOLS_ACCEPTED_LEN 2
#define UC_CHECK_PROTOCOLS_GET_ACCEPTED \
        g_strsplit( \
                        UC_CHECK_PROTOCOLS_ACCEPTED, \
                        UC_SPLIT_DELIMITER, \
                        UC_CHECK_PROTOCOLS_ACCEPTED_LEN \
                  )

#define UC_SPLIT_DELIMITER ","
#define UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_DOCUMENTS "mov,cfm,html,htm,pdf,doc,shtml,asp,php,xml,xls,swf,tar,gz,tgz,zip,bz2,deb,rpm"
#define UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_DOCUMENTS_LEN 19
#define UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_IMAGES "png,gif,jpeg,xpm,bmp"
#define UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_IMAGES_LEN 5
#define UC_CONFIG_PROPERTIES_GET_DEFAULT_CONTENT_TYPES_DOCUMENTS \
        g_strsplit( \
                        UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_DOCUMENTS, \
                        UC_SPLIT_DELIMITER, \
                        UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_DOCUMENTS_LEN \
        )
#define UC_CONFIG_PROPERTIES_GET_DEFAULT_CONTENT_TYPES_IMAGES \
        g_strsplit( \
                       UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_IMAGES, \
                       UC_SPLIT_DELIMITER, \
                       UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_IMAGES_LEN \
        )
#define UC_CONFIG_PROPERTIES_GENERAL_DEFAULT_WORKING_PATH "/tmp/"
#define UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_DOCUMENTS "url/content-types/documents"
#define UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_IMAGES "url/content-types/images"
#define UC_CONFIG_PROPERTIES_URL_NETWORKING_CHECK_LEVEL "url/networking/check-level"
#define UC_CONFIG_PROPERTIES_URL_NETWORKING_DEPTH_LEVEL "url/networking/depth-level"

#define UC_CONFIG_PROPERTIES_GENERAL_DISPLAY_FORMAT "general/display/format"
#define UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_TIMEOUT "general/networking/timeout"
#define  UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_PROXY_CONFIG "general/networking/proxy-config"
#define UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_PROXY_HOST "general/networking/proxy-host"
#define UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_PROXY_PORT "general/networking/proxy-port"
#define UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_PROXY_LOGIN "general/networking/proxy-login"
#define UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_PROXY_PASSWORD "general/networking/proxy-password"
#define UC_CONFIG_PROPERTIES_GENERAL_WORKING_PATH "general/directories/working-path"

#define UC_CONFIG_GET_VECTOR(key, vector, size) \
  gnome_config_get_vector (PROPERTIES_FILENAME "/" key, size, vector)


#define UC_CONFIG_GET_STRING(key) \
        gnome_config_get_string (PROPERTIES_FILENAME "/" key)
#define UC_CONFIG_GET_BOOL(key) \
        gnome_config_get_bool (PROPERTIES_FILENAME "/" key)
#define UC_CONFIG_GET_INT(key) \
        gnome_config_get_int (PROPERTIES_FILENAME "/" key)

#define UC_CONFIG_SET_STRING(key, value) \
        gnome_config_set_string (PROPERTIES_FILENAME "/" key, value)
#define UC_CONFIG_SET_INT(key, value) \
        gnome_config_set_int (PROPERTIES_FILENAME "/"  key, value)
#define UC_CONFIG_SET_VECTOR(key, values, size) \
        gnome_config_set_vector (g_strdup_printf("%s/%s", PROPERTIES_FILENAME, key), size, values)

#define UC_GET_CUSTOM_WIDGET(glade, window, widget_name, widget) \
{ \
  if (!(widget = UC_GET_CUSTOM_WIDGET_REAL(window, widget_name))) \
  { \
    widget = glade_xml_get_widget (glade, widget_name); \
    UC_SET_CUSTOM_WIDGET(window, widget_name, widget); \
  } \
}

#define UC_GET_CUSTOM_WIDGET_REAL(window, widget) \
        (GtkWidget *) g_object_get_data (G_OBJECT (window), widget)

#define UC_SET_CUSTOM_WIDGET(window, name, widget) \
        g_object_set_data_full (G_OBJECT (window), name, gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define UC_LABEL_DISPLAY_IN_TABLE(table, label, name, p1, p2, p3, p4) \
                        gtk_label_set_markup (GTK_LABEL (label), name); \
                                        gtk_table_attach (GTK_TABLE (table), label, p1, p2, p3, p4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0)

#define UC_WIDGET_SET_SENSITIVE(glade_pointer, name, value) \
         gtk_widget_set_sensitive ( \
           glade_xml_get_widget((GladeXML *) glade_pointer, name), \
           value)

#define BUFFER_LEN 2048
#define BUFFER_URL_LEN 2048
#define URL_PORT_MAX_LEN 5
#define HEADER_STATUS_CODE_LEN 3
#define HEADER_STATUS_CODE_FIRST_POS 9

#define MAX_URLS_CHECK \
        (projectState.type == FILE_PROJECT) ? \
                UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_FILES_NETWORKING_CHECK_LEVEL) : \
                UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_CHECK_LEVEL)
#define MAX_CHECK_DEPTH \
        (projectState.type == FILE_PROJECT) ? \
                UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_FILES_NETWORKING_DEPTH_LEVEL) : \
                UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_DEPTH_LEVEL)

#define PROTOCOL_HTTP "http"
#define CONTENT_TYPE "Content-Type"
#define CONTENT_LENGTH "Content-Length"
#define CONTENT_TYPE_HTML "text/html"

#define URL_DEFAULT_PORT "80"

#define UC_SOCKET_OPEN_FORCE_ARGS_USE TRUE
#define UC_SOCKET_OPEN_DEFAULT FALSE

#define SEARCH_STRING_DOCUMENTS "<a href=\""
#define SEARCH_STRING_IMAGES_TAG "<img "
#define SEARCH_STRING_IMAGES_SRC_TAG "src="
#define SEARCH_STRING_IMAGES_TAG_LEN 5
#define SEARCH_STRING_IMAGES_SRC_TAG_LEN 5
#define SEARCH_STRING_DOCUMENTS_LEN 9
#define SEARCH_STRING_MAX_LEN SEARCH_STRING_DOCUMENTS_LEN

#define CHECK_INTERVAL_WAIT 0
#define CHECK_TIMEOUT_DEFAULT 15
#define CHECK_LEVEL_DEFAULT 1000
#define DEPTH_LEVEL_DEFAULT 2
#define PROXY_PORT_DEFAULT 8080
#define CHECK_TIMEOUT \
        UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_TIMEOUT)

#define USER_AGENT "Mozilla/5.0 " PACKAGE "/" VERSION " (Linux)"
#define REFERER "http://savannah.nongnu.org/projects/gurlchecker/"


/* pointers for glade xml file */
GladeXML *glade_main_window;
GladeXML *glade_settings_dialog;
GladeXML *glade_new_project_dialog;
GladeXML *glade_progress_dialog;
GladeXML *glade_view_source_dialog;


/* windows and dialogs pointers */
GtkWidget *main_window;
GtkWidget *new_project_dialog;
GtkWidget *settings_dialog;
GtkWidget *progress_dialog;
GtkWidget *view_source_dialog;


/* treeview for the url list */
GtkTreeView *treeview;


/* treestore for the url treeview list */
GtkTreeStore *treestore;


/* list of all the checked urls */
GList *uc_link_properties_list;


/* enumeration for the different style of
   informations display */
typedef enum
{
  SIMPLE_PRINTING,
  DETAILED_PRINTING,
  DEFAULT_PRINTING
}
UCDisplayFormat;

/*  structure to store project state */
struct UCProjectState
{
  UCDisplayFormat display_format;
  gchar *url;
  guint check_level;
  guint depth_level;
  gchar *working_path;
  gchar *proxy_config;
  gchar *proxy_host;
  guint proxy_port;
}
projectState;


/* structure to store status code properties */
typedef struct
{
  gchar *message;
  gchar *description;
  GdkColor background;
  GdkColor foreground;
}
UCStatusCode;

#endif
