/*
 * Copyright (C) 2002, 2003 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "report.h"


/* structure to store informations during the
   check for managing a project report */
static struct UCProjectReport
{
  guint checkedLinks;
  guint badLinks;
  guint goodLinks;
  guint timedOutLinks;
  gdouble elapsedTime;
}
projectReport;


static void uc_report_set_label (const gchar * ref_name, gchar * value);


/*
 * set "good links" value
 */
void
uc_report_set_goodlinks (guint val)
{
  projectReport.goodLinks += (val) ? val : 1;
}


/*
 * set "bad links" value
 */
void
uc_report_set_badlinks (guint val)
{
  projectReport.badLinks += (val) ? val : 1;
}


/*
 * set "timeouts" value
 */
void
uc_report_set_timedoutlinks (guint val)
{
  projectReport.timedOutLinks += (val) ? val : 1;
}


/*
 * set "checked links" value
 */
void
uc_report_set_checkedlinks (void)
{
  projectReport.checkedLinks =
    projectReport.timedOutLinks + projectReport.badLinks +
    projectReport.goodLinks;
}


/*
 * reset the check report structure
 */
void
uc_check_report_reset (void)
{
  projectReport.checkedLinks = 0;
  projectReport.badLinks = 0;
  projectReport.goodLinks = 0;
  projectReport.timedOutLinks = 0;
  projectReport.elapsedTime = 0;

  uc_report_display_update ();
}


/*
 * update the display check report
 */
void
uc_report_display_update (void)
{
  guint percent = 0;

  uc_report_set_checkedlinks ();

  uc_report_set_label ("labelReportCheckedLinks",
		       g_strdup_printf ("%d", projectReport.checkedLinks));

  percent =
    (projectReport.checkedLinks) ? (projectReport.badLinks * 100) /
    projectReport.checkedLinks : 0;
  uc_report_set_label ("labelReportBadLinks",
		       g_strdup_printf
		       ("%d <span background=\"%s\"> %d%% </span>",
			projectReport.badLinks, BAD_LINK_BGCOLOR, percent));
  percent =
    (projectReport.checkedLinks) ? (projectReport.goodLinks * 100) /
    projectReport.checkedLinks : 0;
  uc_report_set_label ("labelReportGoodLinks",
		       g_strdup_printf
		       ("%d <span background=\"%s\"> %d%% </span>",
			projectReport.goodLinks, GOOD_LINK_BGCOLOR, percent));
  percent =
    (projectReport.checkedLinks) ? (projectReport.timedOutLinks * 100) /
    projectReport.checkedLinks : 0;
  uc_report_set_label ("labelReportTimedOutLinks",
		       g_strdup_printf
		       ("%d <span background=\"%s\"> %d%% </span>",
			projectReport.timedOutLinks, TIMEOUT_LINK_BGCOLOR,
			percent));
  if (!projectReport.checkedLinks)
    uc_report_set_label ("labelReportElapsedTime", g_strdup ("00:00:00"));

  uc_application_update_ui ();
}

/*
 * display a value in the check report frame
 * caller must allocate memory for the "value" arg as it is freed
 * at function end
 */
static void
uc_report_set_label (const gchar * ref_name, gchar * value)
{
  gtk_label_set_markup (GTK_LABEL
			(glade_xml_get_widget (glade_main_window, ref_name)),
			value);
  g_free (value);
}

/*
 * callback called to display the elapsed
 * time of a check
 */
gint
uc_report_timer_callback (gpointer timer)
{
  time_t t;
  struct tm *td;

  time (&t);
  t -= ((time_t) timer + 3600);
  td = localtime (&t);

  uc_report_set_label ("labelReportElapsedTime",
		       g_strdup_printf ("%02d:%02d:%02d", td->tm_hour,
					td->tm_min, td->tm_sec));

  return 1;
}
