/***************************************************************************
                          birthdayoptions.cpp  -  description
                             -------------------
    begin                : Fri Feb 22 2002
    copyright           : (C) 2002,2003 by Jan Hambrecht
    email                :  jaham@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "birthdayoptions.h"
#include <ksimpleconfig.h>

BirthdayOptions::BirthdayOptions()
{
	m_daysComing = 0;
	m_daysPassed = 0;
	m_showOnStartup = true;
	m_showAnniversary = false;
	m_daysHighlight = 0;
	m_colorHighlight = QColor( 255, 0, 0); //red
}

BirthdayOptions::~BirthdayOptions()
{
}

/** set notification time in days for coming events */
void BirthdayOptions::setDaysComing( uint days )
{
	m_daysComing = days;
}

/** get notification time in days for passed events*/
uint BirthdayOptions::getDaysPassed()
{
	return m_daysPassed;
}

/** set notification time in days for passed events */
void BirthdayOptions::setDaysPassed( uint days )
{
	m_daysPassed = days;
}

/** get notification time in days for coming events*/
uint BirthdayOptions::getDaysComing()
{
	return m_daysComing;
}

/** set flag for notifying birthdays on startup */
void BirthdayOptions::setShowOnStartup( bool value )
{
	m_showOnStartup = value;
}

/** get flag for notifying birthdays on startup */
bool BirthdayOptions::getShowOnStartup()
{
	return m_showOnStartup;
}

/** set flag for notifying anniversaries */
void BirthdayOptions::setShowAnniversary( bool value )
{
	m_showAnniversary = value;
}

/** get flag for notifying anniversaries */
bool BirthdayOptions::getShowAnniversary()
{
	return m_showAnniversary;
}               

/** load configuration options */
void BirthdayOptions::load()
{
	KSimpleConfig *config = new KSimpleConfig( "kbirthdayrc", false );
	m_daysComing = config->readNumEntry( "daysComing", 7 );
	m_daysPassed = config->readNumEntry( "daysPassed", 0 );
	m_showOnStartup = config->readBoolEntry( "showOnStartup", true );
	m_showAnniversary = config->readBoolEntry( "showAnniversary", false );
	m_daysHighlight = config->readNumEntry( "daysHighlight", 2 );
	m_colorHighlight = config->readColorEntry( "colorHighlight", new QColor( 255, 0, 0 ) );
	delete config;
}

/** save configuration options */
void BirthdayOptions::save()
{
	KSimpleConfig *config = new KSimpleConfig( "kbirthdayrc" );
	config->writeEntry( "daysComing", m_daysComing );
	config->writeEntry( "daysPassed", m_daysPassed );
	config->writeEntry( "showOnStartup", m_showOnStartup );
	config->writeEntry( "showAnniversary", m_showAnniversary );
	config->writeEntry( "daysHighlight", m_daysHighlight );
	config->writeEntry( "colorHighlight", m_colorHighlight );
	delete config;
}

/** get time in days for highlighting*/
uint BirthdayOptions::getDaysHighlight()
{
	return m_daysHighlight;
}

/** set time in days for highlighting*/
void BirthdayOptions::setDaysHighlight( uint days )
{
	m_daysHighlight = days;
}

/** get highlighting color */
QColor BirthdayOptions::getHighlightColor()
{
	return m_colorHighlight;
}

/** set highlighting color */
void BirthdayOptions::setHighlightColor( QColor color )
{
	m_colorHighlight = color;
}
