/***************************************************************************
                          birthdayscanner.h  -  description
                             -------------------
    begin                : Tue Feb 26 2002
    copyright           : (C) 2002,2003 by Jan Hambrecht
    email                : jaham@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BIRTHDAYSCANNER_H
#define BIRTHDAYSCANNER_H

#include <kabc/stdaddressbook.h>
#include <qvaluelist.h>
#include <qdatetime.h>

#include "birthdayeventdata.h"
#include "birthdayoptions.h"

using namespace KABC;

typedef QValueList<BirthdayEventData> EntryList;

/**
* this class scans all addressbook entries for occuring events like birthdays and anniversaries
* @author Jan Hambrecht
*/

class BirthdayScanner {
public:
	BirthdayScanner();
	~BirthdayScanner();
	/**
	* scans addressbook entries
	* @return true if any event in given interval was found, else false
	*/
	bool scan();
	/**
	* get events count
	* @param isBirthday set to true to get birthdays count (default), else set to false
	* @return events count
	*/
	uint getEventsCount( bool isBirthday = true );
	/**
	* get next event entry
	* @param index index of entry to return
	* @param event event data get filled in here
	* @param isBirthday set to true to get next birthday (default), else set to false
	* @return true if next entry exists, else false
	*/
	bool getEvent( uint index, BirthdayEventData &event, bool isBirthday );
	/**
	* set scan options
	* @param options scannning options
	*/
	void setScanOptions( BirthdayOptions *options );
	/*
	* get addressee from uid
	* @param uid uid of addressee 
	* @param addressee out parameter for found addressee
	* @return true, if addressee exists, else false
	*/
	bool getAddressee( const QString &uid, Addressee &addressee );
protected:
	/**
	* check if addressbook entry should be selected
	* @param entry addressbook entry to check
	* @param isBirthday checks birthday if set to true (default), else checks anniversary
	*/
	void checkEntry( Addressee &entry, bool isBirthday = true );
	/**
	* fill addressee data into event object
	* @param date event date
	* @param entry addressee data
	* @param isBirthday set to true if event is birthday (default), else set to false
	*/
	void fillEvent( QDate &date, Addressee &entry, bool isBirthday = true );
	/**
	* get anniversary date form addressee
	* @param entry addressee data
	* @return date of anniversary
	*/
	QDate getAnniversary( const Addressee &entry );
private: // Private attributes
	/** all found birthday events */
	EntryList m_birthdays;
	/** all found anniversary events */
	EntryList m_anniversaries;
	/** actual date */
	QDate m_today;
	/** scanning options */
	BirthdayOptions *m_options;
	/** pointer to addressbook */
	AddressBook *m_ab;
};

#endif
