/***************************************************************************
                          kbirthday.cpp  -  description
                             -------------------
    begin                : Thu Oct 11 17:27:47 CEST 2001
    copyright           : (C) 2001-2003 by Jan Hambrecht
    email                : jaham@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kbirthday.h"
#include <kglobal.h>
#include <klocale.h>
#include <kprocess.h>
#include <kmessagebox.h>
#include <qtooltip.h>

#ifdef HAVE_CONFIG_H
    #include <config.h>
#endif

extern "C"
{
	KPanelApplet* init( QWidget *parent, const QString configFile)
	{
		KGlobal::locale()->insertCatalogue("kbirthday");
		return new KBirthday(configFile, KPanelApplet::Normal, 0, parent, "kbirthday");
	}
}

KBirthday::KBirthday(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
       : KPanelApplet(configFile, type, actions, parent, name)
{
	setBackgroundMode(X11ParentRelative);
	// create icon
	m_icon = new QPixmap( birthday );
	setIcon( *m_icon );

	// create popup menu
	m_popup = new KPopupMenu();
	m_popup->insertTitle( *m_icon, "KBirthday" );
	m_popup->insertItem( i18n("&Show Events"), ITEM_ALARM );
	m_popup->insertItem( i18n("Address&book"), ITEM_ADDRESSBOOK );
	m_popup->insertSeparator();
	m_popup->insertItem( i18n("&About"), ITEM_ABOUT );
	m_popup->insertItem(SmallIcon("configure"), i18n("&Preferences..."), ITEM_CONFIG);
	connect( m_popup, SIGNAL(activated(int)), this, SLOT(popupClick(int)));

	// applet button
	m_button = new QToolButton( this );
	m_button->setBackgroundMode(X11ParentRelative);
	m_button->setPixmap( *m_icon );
	m_button->setFixedSize( width(), height() );
	m_button->setOn( FALSE );
	QToolTip::add(m_button, "KBirthday");
	connect( m_button, SIGNAL(clicked()), this, SLOT( showPopup()));

	// about data
	m_about = new KAboutData( "kbirthday", "KBirthday", VERSION,
	I18N_NOOP("KBirthday reminds you of birthdays and anniversaries\nfrom your kde addressbook."),
	KAboutData::License_GPL, "(c) 2002,2003, Jan Hambrecht");
	m_about->addAuthor("Jan Hambrecht", 0, "jaham@gmx.net");
	m_about->setTranslator("Jan Hambrecht", "jaham@gmx.net");

	m_options = new BirthdayOptions();
	m_options->load();
}

KBirthday::~KBirthday()
{
	delete m_icon;
	delete m_popup;
	delete m_button;
	delete m_options;
	delete m_about;
}

void KBirthday::about()
{
	KAboutApplication dlg( m_about );
	dlg.exec();
}

int KBirthday::widthForHeight( int height ) const
{
	return APPLET_SIZE;
}

int KBirthday::heightForWidth( int width ) const
{
	return APPLET_SIZE;
}

void KBirthday::resizeEvent( QResizeEvent *_Event)
{
	m_button->setFixedSize( width(), height() );
}

/** event handler for popup menu click */
void KBirthday::popupClick(int id)
{
	switch ( id )
	{
		case -1:
		break;
		case ITEM_CONFIG :
			showPreferences();
		break;
		case ITEM_ABOUT :
			about();
		break;
		case ITEM_ALARM :
			showEventsDlg();
		break;
		case ITEM_ADDRESSBOOK:
		    launchAddressBook();
		break;
	}
}
/** show configuration dialog */
void KBirthday::showPreferences(void)
{
	ConfDlgImpl *dlg = new ConfDlgImpl( 0, "confdlg", true );
	dlg->setOptions( m_options );
	if( dlg->exec() )
	{
		dlg->getOptions( m_options );
		m_options->save();
	}
	delete dlg;
}

/** event handler for mouse click on applet */
void KBirthday::mousePressEvent(QMouseEvent *e)
{
	if ( e->button() == LeftButton || e->button() == RightButton )
	{
		showPopup();
 	}
}
/** arrange button with icon to cover the whole applet */
void KBirthday::arrangeIcon(void)
{
	m_button->setFixedSize( height(), width() );
}

/** show popup menu */
void KBirthday::showPopup()
{
	QPoint p = mapToGlobal( QPoint( 0, 0 ) );
	p.setY( p.y() - m_popup->sizeHint().height() );
	m_popup->exec( p );
	m_button->setOn( FALSE );
}

/** show alarm dialog */
bool KBirthday::showEventsDlg( bool showMessage )
{
	bool ret;
	ret = m_alarm.init( m_options );
	if( ret )
	{
		if( m_alarm.isHidden() )
			m_alarm.show();
	}
	else
	{
		if( showMessage )
			KMessageBox::information(0, i18n( "No Events in last %1 and next %2 days." ).arg( m_options->getDaysPassed() ).arg( m_options->getDaysComing() ) );
	}
	return ret;
}

/** called when applet is shown */
void KBirthday::showEvent( QShowEvent *e )
{
	if( m_options->getShowOnStartup() )
		showEventsDlg( false );
}

/**
launch kde addressbook
*/
void KBirthday::launchAddressBook()
{
	KProcess proc;
	proc << "kaddressbook";
	if( proc.start() )
	{
		proc.detach();
	}
}
