/*
 * Copyright (C) 2002 Stefan Holst
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: lirc.c,v 1.10 2003/03/18 18:09:02 skaboy Exp $
 *
 * lirc functions
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_LIRC
#include <lirc/lirc_client.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <sys/types.h>
#include <regex.h>

#include "odk.h"
#include "utils.h"

/*
#define LOG
*/

typedef struct {

  odk_input_t class;

  int lirc_fd;
  struct lirc_config *lircconfig;
  odk_t *odk;
  void (*event_handler)(void *data, oxine_event_t *ev);

} lirc_input_t;

#ifdef LOG
#define LIRC_VERBOSE 1
#else
#define LIRC_VERBOSE 0
#endif

/* lirc thread
  config file for lirc is ~/.oxine/lircrc
  prog name = oxine
  possible config are: (up, down, left, right, select, menu, stop, volmute, volup, voldown)
*/

#ifdef HAVE_LIRC
static void *lirc_thread(void *this) {

  lirc_input_t *li = (lirc_input_t*) this;
  char *code, *config;
  int ret, r;
  oxine_event_t ev;
  char file[256];
  regex_t reg;
  regmatch_t match[3];


#ifdef LOG
  printf("lirc: lirc tread created\n");
#endif
  
  if((li->lirc_fd = lirc_init("oxine", LIRC_VERBOSE)) == -1) {

    printf("lirc: lirc_init failed. Make sure that you have lircd running\n");
    printf("lirc: and that you have the permissions to connect to the socket\n");
    
    pthread_exit(NULL);
    return NULL;
  }

  snprintf(file, 255, "%s/.oxine", getenv ("HOME"));
  strcat(file, "/lircrc");

  if(lirc_readconfig(file, &li->lircconfig, NULL) != 0) {
    /* printf("not home lircrc, using default\n"); */
    snprintf(file, 255, "%s/lircrc", OXINE_DATADIR);
 
    if(lirc_readconfig(file, &li->lircconfig, NULL) != 0) {
 
      printf("lirc: lirc_readconfig failed\n");
    
      pthread_exit(NULL);
      return NULL;
    }
  }

  r = regcomp(&reg, "seek ([+-]?[0-9]+)", REG_EXTENDED);

  while(1) {
    
    pthread_testcancel();
    
    while(lirc_nextcode(&code) == 0) {

      pthread_testcancel();

      if(code == NULL) 
	break;
      
      ev.type = OXINE_EVENT_KEY;
      ev.key = 0;
      while((ret = lirc_code2char(li->lircconfig, code, &config) == 0) && (config != NULL))      {
#ifdef LOG
      printf("lirc: received command: '%s'\n", config);
#endif
	if (!strcmp(config, "up"))
	  ev.key = OXINE_KEY_UP;
	else if(!strcmp(config, "down"))
	  ev.key = OXINE_KEY_DOWN;
	else if(!strcmp(config, "left"))
	  ev.key = OXINE_KEY_LEFT;
	else if(!strcmp(config, "right"))
	  ev.key = OXINE_KEY_RIGHT;
	else if(!strcmp(config, "select"))
	  ev.key = OXINE_KEY_SELECT;
	else if(!strcmp(config, "menu"))
	  ev.key = OXINE_KEY_OSD;
	else if(!strcmp(config, "stop"))
	  ev.key = OXINE_KEY_STOP;
	else if(!strcmp(config, "pplay"))
	  ev.key = OXINE_KEY_PPLAY;
	else if(!strcmp(config, "next"))
	  ev.key = OXINE_KEY_PL_NEXT;
	else if(!strcmp(config, "prev"))
	  ev.key = OXINE_KEY_PL_PREV;
	else if(!strcmp(config, "volup"))
	  ev.key = OXINE_KEY_VOLUP;
	else if(!strcmp(config, "voldown"))
	  ev.key = OXINE_KEY_VOLDOWN;	  
	else if(!strcmp(config, "volmute"))
	  ev.key = OXINE_KEY_VOLMUTE;	  
	else if(!strcmp(config, "eject"))
	  ev.key = OXINE_KEY_EJECT;	  
	else if(!strncmp(config, "seek", 4)){
	  char str[255];	 
	  r = regexec(&reg, config, 3, match, 0);	  
	  memset(str, 0, 255);
	  strncpy(str, config + match[1].rm_so, match[1].rm_eo - match[1].rm_so);
	  /*printf("%s %d\n", str, r);*/
	  ev.data = strdup(str);
	  ev.key = OXINE_KEY_SEEK;
	}

	if (ev.key) li->event_handler(li->odk, &ev);

      }
      pthread_testcancel();
      
      free(code);

      if(ret == -1) 
	break;
    }
  }

  pthread_exit(NULL);
  return NULL;
}
#endif /* HAVE_LIRC */

odk_input_t *start_lirc(odk_t *odk, void (*event_handler)(void *data, oxine_event_t *ev)) {
#ifdef HAVE_LIRC
  lirc_input_t *li = ho_new_tagged(lirc_input_t, "lirc object");
  li->odk = odk;
  li->event_handler = event_handler;
  pthread_create(&li->class.thread, NULL, lirc_thread, li);

  return (odk_input_t*) li;
#else
#ifdef LOG
  printf("lirc: lirc support disabled at compiletime\n");
#endif
  return NULL;
#endif
}

void stop_lirc(odk_input_t *oi) {
#ifdef HAVE_LIRC
  lirc_input_t *li = (lirc_input_t*) oi;
  pthread_cancel(oi->thread);
  ho_free(li);
#endif
}


