/* 
 * Copyright (C) 2003 the oxine project
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: list.h,v 1.6 2003/03/20 20:25:46 holstsn Exp $
 *
 * doubly linked lists with builtin iterator,
 * based upon xine_list functions of the xine project
 */


#ifndef HAVE_LIST_H
#define HAVE_LIST_H

#include <inttypes.h>

typedef struct list_s list_t;

/*
 * create a new, empty list
 */

#ifdef DEBUG
list_t *_list_new_tagged(char*, int) ;
#define list_new() (_list_new_tagged(__FILE__, __LINE__))
#define xine_list_new() (_list_new_tagged(__FILE__, __LINE__))
#else
list_t *list_new (void) ;
#define xine_list_new list_new
#endif

/*
 * dispose a list (and only the list, contents have to be managed separately)
 */

void list_free (list_t *l) ;

/*
 * set cursor to the first element in list, return contents of first entry
 */

void *list_first_content (list_t *l) ;

/*
 * set cursor to the next element, returns its contents
 */

void *list_next_content (list_t *l) ;

/*
 * returns contents of the current element
 */

void *list_current_content (list_t *l) ;

/*
 * returns 1, if list is empty
 */

int list_is_empty (list_t *l) ;

/*
 * set cursor to the last element, returns its contents
 */

void *list_last_content (list_t *l) ;

/*
 * set cursor to the previous element, return its contents
 */

void *list_prev_content (list_t *l) ;

/*
 * sort element into list by priority. list_first_content returns then
 * the element with the highest priority.
 */

void list_append_priority_content (list_t *l, void *content, int priority) ;

/*
 * append element at the end of the list
 */

#ifdef DEBUG
void _list_append_content (list_t *l, void *content, char*, int) ;
#define list_append_content(list,cont) (_list_append_content(list, cont, __FILE__, __LINE__))
#define xine_list_append_content(list,cont) (_list_append_content(list, cont, __FILE__, __LINE__))
#else
void list_append_content (list_t *l, void *content) ;
#define xine_list_append_content list_append_content
#endif

/*
 * insert content just before cursor position
 */

void list_insert_content (list_t *l, void *content) ;

/*
 * delete current element (its contents have to be freen seperately!)
 */

void list_delete_current (list_t *l) ;

/*
 * compatibility macros
 */

#define xine_list_t list_t
#define xine_list_free list_free
#define xine_list_first_content list_first_content
#define xine_list_next_content list_next_content
#define xine_list_current_content list_current_content
#define xine_list_is_empty list_is_empty
#define xine_list_last_content list_last_content
#define xine_list_prev_content list_prev_content
#define xine_list_append_priority_content list_append_priority_content
#define xine_list_insert_content list_insert_content
#define xine_list_delete_current list_delete_current

#endif
