/*
 * Copyright (C) 2002-2003 Stefan Holst
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: odk_plugin.h,v 1.2 2003/03/19 15:50:31 holstsn Exp $
 *
 * odk header file for window and input plugins
 */

#ifndef HAVE_ODK_PLUGIN_H
#define HAVE_ODK_PLUGIN_H

#include "event.h"

typedef struct odk_window_s odk_window_t;

/*
 * plugin description
 */

typedef struct {

  const char *name;    /* short plugin name */
  const char *desc;    /* one line of plugin description */
  const char *driver;  /* video driver for this plugin, pass this to constructor! */
  
  odk_window_t *(*constructor)(xine_t *xine, const char *driver);
  
} window_plugin_desc_t;

/*
 * window base class
 */

typedef void (*odk_cb) (void *data, oxine_event_t *ev);

struct odk_window_s {

  /* set by odk: */
  
  xine_stream_t           *stream;
  xine_audio_port_t       *audio_port;

  /* set by derived classes: */
  
  int                     xpos, ypos;
  int                     width, height;

  xine_video_port_t       *video_port;

  xine_t                  *xine;

  int  (*show)              (odk_window_t *this);
  int  (*hide)              (odk_window_t *this);
  int  (*displace)          (odk_window_t *this, int x, int y, int w, int h);
  void (*event_loop)        (odk_window_t *this);
  void (*stop_event_loop)   (odk_window_t *this);
  int  (*dispose)           (odk_window_t *this);
  int  (*fullscreen)        (odk_window_t *this, int mode);
  void (*set_event_handler) (odk_window_t *this, odk_cb cb, void *data);
};


#endif
