/*
 * Copyright (C) 2002-2003 Stefan Holst
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: oxine.c,v 1.66 2003/03/23 21:23:24 holstsn Exp $
 *
 * oxine main program
 */

#include "config.h"

#include <stdio.h>
#include <ctype.h>
#include <dirent.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <xine.h>
#include "oxine.h"
#include "odk.h"
#include "otk.h"
#include "event.h"
#include "utils.h"
#include "disc.h"
#include "mediamarks.h"
#include "playlist.h"
#include "controlpc.h"

/*
#define LOG
*/

#define MENU_BG_DEFAULT "%s/menu_bg.mpg"

static void main_menu_cb(void *data);
static void playing_menu_cb(void *data);
static void media_stop_cb(void *data);
static void media_info_cb(void *data);
//static void media_returnto_cb(void *data);
//static void media_freeandreturnto_cb(void *data);
static void media_pause_cb(void *data, int i);

//static void event_delay(void *data);

static void event_handler(void *data, oxine_event_t *ev) {

  oxine_t *oxine = (oxine_t*) data;

  if (ev->type == OXINE_EVENT_KEY) {

    switch(ev->key) {

      case OXINE_KEY_STOP:
	media_stop_cb(oxine);
	ev->key = OXINE_KEY_NULL;
	return;
	break;
    case OXINE_KEY_PLAY:
    case OXINE_KEY_PPLAY:
	switch(odk_get_speed(oxine->odk)) {
	  case ODK_SPEED_NORMAL:
	    odk_set_speed(oxine->odk, ODK_SPEED_PAUSE);
	    if (oxine->pauseplay) otk_selector_set(oxine->pauseplay, 2);
	    break;
	  case ODK_SPEED_PAUSE:
	    odk_set_speed(oxine->odk, ODK_SPEED_NORMAL);
	    if (oxine->pauseplay) otk_selector_set(oxine->pauseplay, 1);
	    break;
	}
	/* if (oxine->need_draw) otk_draw_all(oxine->otk); */
	ev->key = OXINE_KEY_NULL;
	return;
	break;
    case OXINE_KEY_PL_NEXT:
      {
	char *mrl, *title;
	playlist_get_next(oxine->playlist, &mrl, &title);
	odk_set_mode(oxine->odk, ODK_MODE_NORM);
	odk_open(oxine->odk, mrl);
	odk_play(oxine->odk);
	ev->key = OXINE_KEY_NULL;
	return;
	break;
      }
    case OXINE_KEY_PL_PREV:
      {
	char *mrl, *title;
	playlist_get_prev(oxine->playlist, &mrl, &title);
	odk_set_mode(oxine->odk, ODK_MODE_NORM);
	odk_open(oxine->odk, mrl);
	odk_play(oxine->odk);
	ev->key = OXINE_KEY_NULL;
	return;
	break;
      }

    case OXINE_KEY_PAUSE:
	media_stop_cb(oxine);
	ev->key = OXINE_KEY_NULL;
	return;
	break;
      case OXINE_KEY_EJECT:
	disc_eject(oxine);
	ev->key = OXINE_KEY_NULL;
	break;
      case OXINE_KEY_OSD:
	playing_menu_cb(data);
	ev->key = OXINE_KEY_NULL;
	break;
      case OXINE_KEY_VOLMUTE:
	odk_volmute(oxine->odk);
	/* if (oxine->need_draw) otk_draw_all(oxine->otk); */
	ev->key = OXINE_KEY_NULL;
	return;
	break;
      case OXINE_KEY_VOLUP:
	odk_set_volume(oxine->odk, 1);
	/* if (oxine->need_draw) otk_draw_all(oxine->otk); */
	ev->key = OXINE_KEY_NULL;
	return;
	break;
      case OXINE_KEY_VOLDOWN:
	odk_set_volume(oxine->odk, -1);
 	/* if (oxine->need_draw) otk_draw_all(oxine->otk); */
	ev->key = OXINE_KEY_NULL;
	return;
	break;
    case OXINE_KEY_SEEK:
      odk_seek(oxine->odk, atoi(ev->data));
      if(ev->data) free(ev->data);
      /* if (oxine->need_draw) otk_draw_all(oxine->otk); */
      ev->key = OXINE_KEY_NULL;
      return;
      break;
    }
  } else if (ev->type == OXINE_EVENT_FINISH) {

    char *mrl, *title;

    oxine->cd_in_use = 0;
    oxine->pauseplay = NULL;
    oxine->main_window = NULL;
    otk_clear(oxine->otk);
    if (oxine->reentry) {
      odk_set_mode(oxine->odk, ODK_MODE_LOGO);
      odk_open(oxine->odk, oxine->menu_bg);
      odk_play(oxine->odk);
      oxine->reentry(oxine->reentry_data);
    } else {
      if (playlist_get_first_unplayed(oxine->playlist, &mrl, &title)) {
	odk_set_mode(oxine->odk, ODK_MODE_NORM);
	odk_open(oxine->odk, mrl);
	odk_play(oxine->odk);
      } else {
	odk_set_mode(oxine->odk, ODK_MODE_LOGO);
	odk_open(oxine->odk, oxine->menu_bg);
	odk_play(oxine->odk);
        oxine->main_menu_cb(oxine);
      }
    }
  }
  else if(ev->type == OXINE_EVENT_START) {
    if(!odk_is_logo_mode(oxine->odk))
      oxine->blending_job=schedule_job(3000, media_info_cb, oxine);
    ho_free(ev);
  }
  
}

/*
static void event_delay(void *data) {
  oxine_event_t *ev = (oxine_event_t *) data;
  odk_t *odk = (odk_t *) ev->data;

  ev->data = NULL;
  odk_send_event(odk, ev);
}
*/

const char *playpause_strings[] = { "<", ">" };

static void media_pause_cb(void *data, int i) {
  oxine_t *oxine = (oxine_t*) data;
 
  switch(i) {
    case 1:
      odk_set_speed(oxine->odk, ODK_SPEED_NORMAL);
      break;
    case 2:
      odk_set_speed(oxine->odk, ODK_SPEED_PAUSE);
      break;
  }
}

static void format_time(char *buf, int sec) {
    sprintf(buf, "%d:%02d:%02d", sec/3600, (sec%3600)/60, ((sec%3600)%60));
}

static void media_stop_cb(void *data) {
  oxine_t *oxine = (oxine_t*) data;

  odk_stop(oxine->odk);
  odk_set_mode(oxine->odk, ODK_MODE_LOGO);
  odk_open(oxine->odk, oxine->menu_bg);
  odk_play(oxine->odk);
  if (oxine->reentry)
    oxine->reentry(oxine->reentry_data);
  else
    oxine->main_menu_cb(oxine);
}

static void media_tracks_list_cb(void *data, void *entry) {
  oxine_t *oxine = (oxine_t *) data;
  char *mrl = (char *) entry;

  odk_stop(oxine->odk);
  odk_open(oxine->odk, mrl);
  odk_play(oxine->odk);
}

/*
static void media_tracks_quit(void *data) {
  oxine_t *oxine = (oxine_t*) data;

  oxine->need_draw = 0;
  printf("%d\n", oxine->mode);
  oxine->mode = OXINE_MODE_NORMAL;
  printf("%d\n", oxine->mode);
  oxine->pauseplay = NULL;
  otk_clear(oxine->otk); 
}
*/

static void media_tracks_cb(void *data) {
  oxine_t *oxine = (oxine_t *) data;
  int num, i = 0;
  char **str;
  otk_widget_t *list;

  oxine->pauseplay = NULL;
  oxine->main_window = NULL;
  oxine->info_window = NULL;
  otk_clear(oxine->otk);
  oxine->main_window = otk_window_new (oxine->otk, NULL, 50, 50, 550, 500);
  list = otk_list_new(oxine->main_window, 10, 10, 500, 490, media_tracks_list_cb, oxine);

  str = xine_get_autoplay_mrls (oxine->xine, "CD", &num);
  while(i < num) {
    /* printf("%d %s\n", i, str[i]); */
    otk_add_listentry(list, str[i], str[i], -1);
    i++;
  } 
  otk_list_set_pos(list, 0);
  otk_set_focus(list);
  otk_draw_all(oxine->otk);
}

#if 0
static void info_button_time(void *data) {
  oxine_t *oxine = (oxine_t *)data;
  int ret, pos_time, length;
  
  ret = odk_get_pos_length_high(oxine->odk, NULL, &pos_time, &length);  
  /* oxine->time = malloc(sizeof(char)*255); */
  if(ret) {
    pos_time /= 1000;
    length /= 1000;
    sprintf(oxine->time, "(%d:%02d:%02d / %d:%02d:%02d)", pos_time/3600, (pos_time%3600)/60, ((pos_time%3600)%60), 
	    length/3600, (length%3600)/60, ((length%3600)%60));
  }
  else sprintf(oxine->time, "N/A");
}
#endif

static void media_info_close_cb(void *data) {
  oxine_t *oxine = (oxine_t*) data;

  if (!oxine->info_window) return;
  
  otk_destroy_widget(oxine->info_window);
  oxine->info_window = NULL;
  otk_draw_all(oxine->otk);

  if (oxine->lines[0]) ho_free(oxine->lines[0]);
  if (oxine->lines[1]) ho_free(oxine->lines[1]);
  if (oxine->lines[2]) ho_free(oxine->lines[2]);
}

static void media_info_cb(void *data) {
  oxine_t *oxine = (oxine_t*) data;
  int pos_time, length, ret;
  int cline = 0;
  char *buf1, *buf2;
  /* otk_widget_t *b, *layout; */
  oxine->blending_job = 0;

  oxine->media_info_close_cb = media_info_close_cb;
  oxine->pauseplay = NULL;
  if (oxine->info_window) {
    otk_destroy_widget(oxine->info_window);
    oxine->info_window = NULL;
    otk_draw_all(oxine->otk);
    return;
  }
  /* otk_clear(oxine->otk); */

  buf2 = NULL;
  buf1 = odk_get_mrl(oxine->odk);
  if (buf1) {
    buf2 = strrchr(buf1,'/');
    if (buf2) buf2++; else buf2 = buf1;
  }

  if (oxine->lines[0]) ho_free(oxine->lines[0]);
  if (oxine->lines[1]) ho_free(oxine->lines[1]);
  if (oxine->lines[2]) ho_free(oxine->lines[2]);
  
  oxine->lines[0] = odk_get_meta_info(oxine->odk, XINE_META_INFO_TITLE);
  if(!oxine->lines[0] && buf2) oxine->lines[0] = ho_strdup(buf2);
  if(oxine->lines[0]) cline++;
  buf1 = odk_get_meta_info(oxine->odk, XINE_META_INFO_ARTIST);
  buf2 = odk_get_meta_info(oxine->odk, XINE_META_INFO_ALBUM);
  if (buf1 && buf2) {
    oxine->lines[cline] = ho_newstring(strlen(buf1)+strlen(buf2)+10);
    sprintf(oxine->lines[cline], "%s: %s", buf1, buf2);
    cline++;
  } else if (buf1 && (strlen(buf1)>0)) {
    oxine->lines[cline++] = ho_strdup(buf1);
  } else if (buf2 && (strlen(buf2)>0)) {
    oxine->lines[cline++] = ho_strdup(buf2);
  }
  /*
  oxine->genre = odk_get_meta_info(oxine->odk, XINE_META_INFO_GENRE);
  if(!oxine->genre) oxine->genre = ho_strdup("Genre N/A");
  oxine->year = odk_get_meta_info(oxine->odk, XINE_META_INFO_YEAR);
  if(!oxine->year) oxine->year = ho_strdup("Year N/A");
*/
  ret = odk_get_pos_length_high(oxine->odk, NULL, &pos_time, &length);  

  if(ret && (length > 0)) {
    oxine->lines[cline] = ho_newstring(255);
    pos_time /= 1000;
    length /= 1000;
    format_time(oxine->lines[cline], length);
  }

  oxine->info_window = otk_window_new (oxine->otk, NULL, 5, 5, 790, 240);

  if (oxine->lines[0])
  otk_label_new(oxine->info_window, 5, 40, OTK_ALIGN_LEFT|OTK_ALIGN_VCENTER, oxine->lines[0]);
  
  if (oxine->lines[1])
  otk_label_new(oxine->info_window, 5, 110, OTK_ALIGN_LEFT|OTK_ALIGN_VCENTER, oxine->lines[1]);
  
  if (oxine->lines[2])
  otk_label_new(oxine->info_window, 5, 180, OTK_ALIGN_LEFT|OTK_ALIGN_VCENTER, oxine->lines[2]);
  
  /*
  layout = otk_layout_new(oxine->info_window, 10, 10, 680, 480, 6, 1);

  b = otk_button_grid_new(oxine->title, media_freeandreturnto_cb, oxine);  
  otk_layout_add_widget(layout, b, 0, 0, 1, 1);
  otk_set_focus(b);
 
  b = otk_button_grid_new(oxine->artist, media_freeandreturnto_cb, oxine);  
  otk_layout_add_widget(layout, b, 0, 1, 1, 1);

  b = otk_button_grid_new(oxine->genre, media_freeandreturnto_cb, oxine);  
  otk_layout_add_widget(layout, b, 0, 2, 1, 1);
   
  b = otk_button_grid_new(oxine->album, media_freeandreturnto_cb, oxine);  
  otk_layout_add_widget(layout, b, 0, 3, 1, 1);
 
  b = otk_button_grid_new(oxine->year, media_freeandreturnto_cb, oxine);  
  otk_layout_add_widget(layout, b, 0, 4, 1, 1);
  
  b = otk_button_grid_new(oxine->time, media_freeandreturnto_cb, oxine);  
  otk_layout_add_widget(layout, b, 0, 5, 1, 1);
  otk_button_uc_set(b, info_button_time, oxine);
 */
  
  otk_draw_all(oxine->otk);

  schedule_job(5000, media_info_close_cb, oxine);
}

/*
static void media_returnto_cb(void *data) {
  oxine_t *oxine = (oxine_t*) data;

  oxine->need_draw = 0;
  printf("%d\n", oxine->mode);
  oxine->mode = OXINE_MODE_NORMAL;
  printf("%d\n", oxine->mode);
  oxine->pauseplay = NULL;
  otk_clear(oxine->otk); 
}
*/

static void shutdown_cb (void *data) {
  oxine_t *oxine = (oxine_t*) data;

  odk_exit(oxine->odk);
}

static void dvb_cb (void *data) {
  oxine_t *oxine = (oxine_t*) data;

  oxine->pauseplay = NULL;
  oxine->main_window = NULL;
  otk_clear(oxine->otk);
  odk_open(oxine->odk, "dvb://");
  odk_set_mode(oxine->odk, ODK_MODE_NORM);
  odk_play(oxine->odk);
}

static void tv_cb (void *data) {
  oxine_t *oxine = (oxine_t*) data;

  oxine->pauseplay = NULL;
  oxine->main_window = NULL;
  otk_clear(oxine->otk);
  odk_open(oxine->odk, "v4l://");
  odk_set_mode(oxine->odk, ODK_MODE_NORM);
  odk_play(oxine->odk);

}

static void controlpc_clearall(void *data) {
  controlpc_cbi_t *info = (controlpc_cbi_t *) data;
  oxine_t *oxine = info->oxine;
  otk_button_cb_t cb = info->quit;

  ho_free(info);
  cb(oxine);
}

static void control_cb (void *data) {
  oxine_t *oxine = (oxine_t*) data;
  otk_widget_t *b, *layout;
  controlpc_cbi_t *info = ho_new(controlpc_cbi_t);

  oxine->pauseplay = NULL;
  oxine->main_window = NULL;
  otk_clear(oxine->otk);
   
  oxine->main_window = otk_window_new(oxine->otk, NULL, 50, 50, 700, 500);
  layout = otk_layout_new(oxine->main_window, 10, 10, 640, 480, 4, 1);

  info->oxine = oxine;
  info->cb = control_cb;
  info->quit = main_menu_cb;

  b = otk_button_grid_new("Health check", controlpc_health_check, info);  
  otk_layout_add_widget(layout, b, 0, 0, 1, 1);
  otk_set_focus(b);
  /* otk_button_new (oxine->main_window, 10, 10+sizey*1, sizex, sizey, "Mount devices", controlpc_mount_cb, info);  */
  b = otk_button_grid_new("Power managment", controlpc_power_cb, info);
  otk_layout_add_widget(layout, b, 0, 1, 1, 1);
  b = otk_button_grid_new("Main Menu", controlpc_clearall, info);  
  otk_layout_add_widget(layout, b, 0, 3, 1, 1);

  otk_draw_all(oxine->otk);
}


static void playing_menu_update(void *data) {
  oxine_t *oxine = (oxine_t *) data;
  int pos_time, length;

  if (oxine->pos_str)
    if (odk_get_pos_length_high(oxine->odk, NULL, &pos_time, &length)) {
      pos_time /= 1000;
      format_time(oxine->pos_str, pos_time);
    }
}

static void playing_menu_cb(void *data) {
   oxine_t *oxine = (oxine_t *) data;
   otk_widget_t *b;
   otk_widget_t *layout;
   int pos_time, length;
 
   if(odk_is_logo_mode(oxine->odk)) return;
  
   oxine->pauseplay = NULL;
   if (oxine->main_window) {
     otk_destroy_widget(oxine->main_window);
     oxine->main_window = NULL;
     otk_draw_all(oxine->otk);
   }
   
   /* otk_clear(oxine->otk); */

   if(oxine->mode == OXINE_MODE_PLAY_MENU) {
     oxine->mode = OXINE_MODE_NORMAL;
     return ;
   }
   oxine->mode = OXINE_MODE_PLAY_MENU;

   oxine->main_window = otk_window_new (oxine->otk, NULL, 50, 400, 700, 150);
   layout = otk_layout_new(oxine->main_window, 10, 10, 680, 130, 2, 6); 
   oxine->pauseplay = otk_selector_grid_new (playpause_strings, 2, media_pause_cb, oxine);   
   otk_layout_add_widget(layout, oxine->pauseplay, 0, 0, 1, 1);
   otk_set_focus(oxine->pauseplay);
   if (odk_get_speed(oxine->odk) == ODK_SPEED_PAUSE) otk_selector_set(oxine->pauseplay, 2);
   
   b = otk_button_grid_new ("}", media_stop_cb, oxine);  
   otk_layout_add_widget(layout, b, 1, 0, 1, 1);
   
/*   b = otk_button_grid_new ("Volume", NULL, NULL);
   otk_layout_add_widget(layout, b, 0, 1, 1, 1);
 
   b = otk_slider_grid_new (odk_get_volume);
   otk_layout_add_widget(layout, b, 1, 1, 1, 1);

   if(!oxine->cd_in_use) {
     b = otk_button_grid_new("Seek", NULL, NULL);
     otk_layout_add_widget(layout, b, 0, 2, 1, 1);
   }else {
     b = otk_button_grid_new ("T", media_tracks_cb, oxine);
     otk_layout_add_widget(layout, b, 0, 2, 1, 1);
   }*/
   
   if(oxine->cd_in_use) {
     b = otk_button_grid_new ("T", media_tracks_cb, oxine);
     otk_layout_add_widget(layout, b, 3, 0, 1, 1);
   }
   
   b = otk_slider_grid_new (odk_get_seek);
   otk_layout_add_widget(layout, b, 2, 1, 4, 1);

   b = otk_button_grid_new ("i", media_info_cb, oxine);
   otk_layout_add_widget(layout, b, 2, 0, 1, 1);
   
   if (!oxine->pos_str) oxine->pos_str = ho_newstring(64);
   if (odk_get_pos_length_high(oxine->odk, NULL, &pos_time, &length)) {
     pos_time /= 1000;
     format_time(oxine->pos_str, pos_time);
   }
   
   otk_label_new (oxine->main_window,  110, 100, OTK_ALIGN_CENTER|OTK_ALIGN_VCENTER, oxine->pos_str);
   otk_button_uc_set(b, playing_menu_update, oxine);

   /* oxine->need_draw = 1; */

   otk_draw_all(oxine->otk);
}

static void main_menu_cb(void *data) {
  
  oxine_t *oxine = (oxine_t*) data;
  otk_widget_t *b;

  lock_job_mutex();
  if (oxine->blending_job) {
    cancel_job(oxine->blending_job);
  }
  if (oxine->info_window) {
    media_info_close_cb(oxine);
  }
  unlock_job_mutex();

  if (oxine->cd_is_mounted) {
    autocd_umount(oxine->cd_mountpoint);
    oxine->cd_is_mounted = 0;
  }

  oxine->pauseplay = NULL;
  oxine->main_window = NULL;
  otk_clear(oxine->otk);

  oxine->main_window = otk_window_new (oxine->otk, NULL, 50, 130, 700, 420);

  b = otk_button_new (oxine->main_window, 50, 45, 290, 60, "Play Disc", disc_cb, oxine);
  otk_set_focus(b);
  otk_button_new (oxine->main_window, 360, 45, 290, 60, "Mediamarks", mediamarks_cb, oxine);
  
  otk_button_new (oxine->main_window, 50, 150, 290, 60, "Analogue TV", tv_cb, oxine);
  otk_button_new (oxine->main_window, 360, 150, 290, 60, "Playlist", playlist_cb, oxine);

  otk_button_new (oxine->main_window, 50, 255, 290, 60, "Digital TV", dvb_cb, oxine);
  otk_button_new (oxine->main_window, 360, 255, 290, 60, "Control", control_cb, oxine);

  otk_button_new (oxine->main_window, 205, 320, 290, 60, "Shutdown", shutdown_cb, oxine);

/*  otk_button_new (oxine->main_window, 50, 180, 290, 60, "File", file_cb, oxine);
  otk_button_new (oxine->main_window, 50, 260, 290, 60, "Streaming", streaming_cb, oxine);*/

  otk_draw_all(oxine->otk);
}

/*
 * user help
 */

void print_version() {

  printf("This is oxine - a purely osd based xine frontend v%s\n\n", VERSION);
  printf("(c) 2003 Stefan Holst and the oxine project\n");
  printf("This is free software; see the source for copying conditions.  There is NO\n");
  printf("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
}

void print_help() {

  printf("usage: oxine [options] mrls...\n\n");
  printf("options:\n");
  printf("  -v       print version and exit\n");
  printf("  -h       print command line help (this cruft) and exit\n");
  printf("  -f       start in fullscreen mode\n");
  printf("  -V id    video driver id = auto, xv, xshm, fb, dxr3, ...\n");
  printf("  -A id    audio driver id = auto, oss, alsa, ...\n");
  printf("\n");
  printf("For latest info, check http://oxine.sf.net.\n");
}

/*
 * parses user arguments
 */

int parse_arguments(oxine_t *oxine, int argc, char **argv) {
  
  int c;
 
  opterr = 0;
  
  while ((c = getopt (argc, argv, "hvfA:V:")) != -1)
    switch (c)
      {
      case 'v':
        print_version();
	return 0;
        break;
      case 'h':
	print_help();
	return 0;
        break;
      case 'f':
	oxine->fullscreen = 1;
	break;
      case 'A':
        oxine->req_audio_driver = optarg;
	break;
      case 'V':
        oxine->req_video_driver = optarg;
        break;
      case '?':
        if (isprint (optopt))
          fprintf (stderr, "Unknown option `-%c'.\n", optopt);
        else
          fprintf (stderr,
                   "Unknown option character `\\x%x'.\n",
                   optopt);
	return 0;
      default:
	return 0;
      }

  for (c = optind; c < argc; c++)
   playlist_add (oxine->playlist, argv[c], NULL);

  return 1;
}

/*
 * initialisation
 * - alloc oxine struct
 * - parse user arguments
 * - load config file
 * - init xine engine
 */
 
oxine_t *create_oxine(int argc, char **argv) {
  
  oxine_t *oxine;
  char menu_bg_default[1024];
  xine_cfg_entry_t centry;
  
  oxine = ho_new(oxine_t);
  oxine->main_menu_cb = main_menu_cb;
  oxine->playlist = playlist_new();

  if (!parse_arguments(oxine, argc, argv)) {
    playlist_free(oxine->playlist);
    ho_free(oxine);
    return NULL;
  }
  
  oxine->xine = xine_new();
  snprintf (oxine->configfile, 255, "%s/.oxine", getenv ("HOME"));
  mkdir (oxine->configfile, 0700);
  strcat(oxine->configfile, "/config");
  xine_config_load (oxine->xine, oxine->configfile);

  oxine->cd_mountpoint = 
  xine_config_register_string (oxine->xine,
				"oxine.dvd_mountpoint",
				"/dvd",
				"directory a media in dvd device will be mounted",
				"directory a media in dvd device will be mounted",
				10,
				NULL,
				NULL);
  
  snprintf (menu_bg_default, 1023, MENU_BG_DEFAULT, OXINE_DATADIR);

  oxine->menu_bg = 
  xine_config_register_string (oxine->xine,
				"oxine.menu_bg_stream",
				menu_bg_default,
				"stream played in background of osd main menu",
				"stream played in background of osd main menu",
				10,
				NULL,
				NULL);

  xine_config_register_string (oxine->xine,
				"oxine.preferred_video",
				"auto",
				"video driver selected by user",
				"video driver selected by user",
				10,
				NULL,
				NULL);
  
  xine_config_register_string (oxine->xine,
				"oxine.preferred_audio",
				"auto",
				"audio driver selected by user",
				"audio driver selected by user",
				10,
				NULL,
				NULL);

  oxine->mm_sort_type = xine_config_register_num (oxine->xine,
						  "oxine.mm_sort_type",
						  0,
						  "sort type in mediamarks",
						  "sort type in mediamarks",
						  10,
						  NULL,
						  NULL);
  
  xine_init (oxine->xine);

  if (oxine->req_video_driver) {
    xine_config_lookup_entry (oxine->xine, "oxine.preferred_video", &centry);
    centry.str_value = oxine->req_video_driver;
    xine_config_update_entry (oxine->xine, &centry);
  } 

  if (oxine->req_audio_driver) {
    xine_config_lookup_entry (oxine->xine, "oxine.preferred_audio", &centry);
    centry.str_value = oxine->req_audio_driver;
    xine_config_update_entry (oxine->xine, &centry);
  } 

  xine_config_lookup_entry (oxine->xine, "input.dvd_device", &centry);
  oxine->cd_device = centry.str_value;

  return oxine;
}

void destroy_oxine(oxine_t *oxine) {

  xine_config_save (oxine->xine, oxine->configfile);
  xine_exit(oxine->xine);

  playlist_free(oxine->playlist);

  if (oxine->cd_is_mounted) {
    autocd_umount(oxine->cd_mountpoint);
    oxine->cd_is_mounted = 0;
  }

  ho_free(oxine);
}

/*
 * M A I N
 */

int main (int argc, char **argv) {

  oxine_t *oxine = create_oxine(argc,argv);

  if (!oxine) return 0;

  start_scheduler();
  
  oxine->odk = odk_init(oxine->xine);

  if (oxine->odk) {

    odk_listen_to(oxine->odk, "lirc");
    
    oxine->otk=otk_init(oxine->odk);
    
    otk_set_event_handler(oxine->otk, event_handler, oxine);

    oxine->mode = OXINE_MODE_NORMAL;
   
    if (playlist_length(oxine->playlist)>0) {
      char *mrl, *title;
      
      odk_set_mode(oxine->odk, ODK_MODE_NORM);
      playlist_get_first_unplayed(oxine->playlist, &mrl, &title);
      odk_open(oxine->odk, mrl);
      odk_play(oxine->odk);
    }
    else {
      odk_set_mode(oxine->odk, ODK_MODE_LOGO);
      odk_open(oxine->odk, oxine->menu_bg);
      odk_play(oxine->odk);
      main_menu_cb(oxine);
    }

    odk_show_window(oxine->odk);
    odk_fullscreen(oxine->odk, oxine->fullscreen);
    odk_run(oxine->odk);

    if (oxine->otk) otk_free(oxine->otk);
    if (oxine->odk) odk_free(oxine->odk);

  }

  destroy_oxine(oxine);
 
  stop_scheduler();

#ifdef DEBUG
  heapstat();
#endif
  
  return 0;
}
