/*
 * Copyright (C) 2002 Stefan Holst
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: playlist.h,v 1.3 2003/03/18 18:09:02 skaboy Exp $
 *
 * a simple playlist implementataion
 */

#ifndef HAVE_PLAYLIST_H
#define HAVE_PLAYLIST_H

/*
 * opaque playlist data type
 */

typedef struct playlist_s playlist_t;

/*
 * create an empty playlist
 */

playlist_t *playlist_new();

/*
 * append item to playlist, deep copies are made from mrl and title.
 */

void playlist_add(playlist_t *playlist, const char *mrl, const char *title);

/*
 * remove all items from playlist
 */

void playlist_clear(playlist_t *playlist);

/*
 * get first unplayed item from list and mark it as played
 * returns 0 if no unplayed items are available, 1 otherwise.
 */

int playlist_get_first_unplayed(playlist_t *playlist, char **mrl, char **title);

/*
 * mark all items as unplayed
 */

void playlist_reset(playlist_t *playlist);

/*
 * get length of playlist (played and unplayed items are counted)
 */

int playlist_length(playlist_t *playlist);

/*
 * get nth item
 * returns 1 on success, 0 on error.
 */

int playlist_get_nth(playlist_t *playlist, int n, char **mrl, char **title);
/*
 * get current, prev, next selected item
 * and set them selected and played
 */

int playlist_get_next(playlist_t *playlist, char **mrl, char **title);
int playlist_get_prev(playlist_t *playlist, char **mrl, char **title);
int playlist_get_current(playlist_t *playlist);


/*
 * remove nth item from playlist
 * returns 1 on success, 0 on error.
 */

int playlist_remove_nth(playlist_t *playlist, int n);

/*
 * frees given playlist and all its items
 */

void playlist_free(playlist_t *playlist);

/*
 * gui entrance
 */

void playlist_cb (void *data);

#endif
