cvs_id = "$Id: LookupManager.py,v 1.5 2003/03/15 10:36:52 juri Exp $"

import time
import socket
import random
import adns, ADNS

class MyQE(ADNS.QueryEngine):
    def lookup_a_record(self, name, callback, extra):
        self.submit(name, adns.rr.A, callback = callback, extra = extra)

class LookupManager:
    namecache = {}
    queryengine = MyQE()
    
    def __init__(self, mintime=86400, maxtime=259200):
        self.mintime = mintime
        self.maxtime = maxtime

    def _lookup(self, name, callback, cbw):
        now = time.time()
        ip = self.namecache.get(name, None)
        expiretime = 0
        if ip is not None:
            ip, expiretime = ip
            if now < expiretime:
                cbw.cb(ip)
                return
        self.queryengine.lookup_a_record(name, callback, cbw)

    def lookup(self, name, callback, data=None):
        cbw = CBWrapper(name, callback, data)
        self._lookup(name, self.adns_callback, cbw)
        
    def adns_callback(self, answer, qname, rr, flags, cbwrapper):
        now = int(time.time())
        ips = answer[3]
        ip = None
        if answer[1] and not ips:
            # we got a cname even though we asked for an a record
            self._lookup(answer[1], self.adns_callback, cbwrapper)
            return
        elif ips:
            ip = ips[0]
            self.namecache[qname] = (ip, now + random.randint(
                now+self.mintime, now+self.maxtime))
        cbwrapper.cb(ip)

    def poll(self, timeout=0.1):
        self.queryengine.run(timeout)

class CBWrapper:
    def __init__(self, name, callback, data):
        self.name = name
        self.data = data
        self.callback = callback

    def cb(self, ip):
        self.callback(self.name, ip, self.data)

