cvs_id = "$Id: Main.py,v 1.24 2003/03/16 11:48:19 juri Exp $"

import straw

import os
import sys

import pygtk
pygtk.require('2.0')

import gettext
import gnome
import gnome.ui
import gobject
import gtk
import gtkhtml2
import locale
import pango
from gtk import glade
import getopt
from error import log

class Main:
    def __init__(self):
        lname = straw.APPNAME.lower()
        gnome.init(lname, straw.VERSION)
        straw.main = self
        try:
            locale.setlocale(locale.LC_ALL, '')
            localedir = find_locale_dir()
            gettext.bindtextdomain(lname, localedir)
            gettext.textdomain(lname)
            gettext.install(lname, localedir, unicode=1)
            glade.bindtextdomain(lname, localedir)
        except IOError:
            def broken_gettext_workaround(s):
                return s
            __builtins__.__dict__['_'] = broken_gettext_workaround
        libdir = find_lib_dir()
        self._libdir = libdir
        xmlfile = find_glade_file(libdir)
        straw.ImageCache.initialize()
        self._image_loader = ImageLoader(libdir)
        self._visible_feed = None
        self._visible_item = None
        self._user_messages = []
        straw.lookup_manager = straw.LookupManager()
        straw.config = straw.Config()
        if straw.config.first_time:
            try:
                self.import_opml(
                    os.path.join(libdir, "default_subscriptions.opml"))
            except:
                pass
        saver = straw.ConfigSaver()
        try:
            optlist, args = getopt.getopt(sys.argv[1:], '', ['offline'])
            if '--offline' in [o[0] for o in optlist]:
                straw.config.offline = 1
        except getopt.GetoptError, e:
            log("Error parsing the command line: ", e)
        straw.main_window = straw.MainWindow(
            glade.XML(xmlfile, "straw_main", gettext.textdomain()))
        self.update_feeds(straw.config.feeds)
        straw.config.signal_connect(
            straw.FeedsChangedSignal, self.feeds_changed)
        self._itemstore = straw.ItemStore()
        self.restore_feeds(self._itemstore)
        self._preferences_dialog = straw.PreferencesDialog(
            glade.XML(xmlfile, "preferences_dialog", gettext.textdomain()))
        self._subscribe_dialog = straw.SubscribeDialog(
            glade.XML(xmlfile, 'subscribe_dialog', gettext.textdomain()))
        self._find_dialog = straw.FindDialog(
            glade.XML(xmlfile, 'find_dialog', gettext.textdomain()))
        self._subscriptions_merge_dialog = straw.SubscriptionsMergeDialog(
            glade.XML(xmlfile, 'subscriptions_merge_dialog',
                      gettext.textdomain()))
        self._export_subscriptions_dialog = straw.ExportSubscriptionsDialog(
            glade.XML(xmlfile, 'export_subscriptions_dialog',
                      gettext.textdomain()))
        self._import_subscriptions_dialog = straw.ImportSubscriptionsDialog(
            glade.XML(xmlfile, 'import_subscriptions_dialog',
                      gettext.textdomain()))
        self._itemstore.start()
        self._feed_selection_enabled = 1
        straw.poll_manager = straw.PollManager()
        gtk.timeout_add(1000, self.display_messages)
        gtk.timeout_add(
            straw.NetworkConstants.POLL_INTERVAL, self.poll_network)
        return

    def restore_feeds(self, itemstore):
        for f in straw.config.feeds:
            f.restore_items(itemstore.read_feed_items(f))

    def poll_network(self):
        if not straw.config.offline:
            straw.URLFetch.connection_manager.poll(
                straw.NetworkConstants.POLL_TIMEOUT)
        gtk.timeout_add(
            straw.NetworkConstants.POLL_INTERVAL, self.poll_network)

    # called when FeedsChangedSignal is sent
    def feeds_changed(self, signal):
        self.update_feeds(signal.sender.feeds)
        return

    def update_feeds(self, feeds):
        straw.main_window.display_feeds(straw.config.feeds, self._visible_feed)
        return

    def display_feeds(self):
        straw.main_window.display_feeds(straw.config.feeds, self._visible_feed)
        return

    def get_visible_feed(self):
        return self._visible_feed

    def catch_up_feed(self, feed = None):
        if feed is None:
            feed = self._visible_feed
        feed.mark_all_read()

    def display_feed(self, feed, select_first = 1):
        if self._feed_selection_enabled and self._visible_feed != feed:
            self._visible_feed = feed
            if len(feed.items) == 0:
                straw.main_window.display_empty_feed(feed)
            else:
                straw.main_window.display_feed(feed, select_first)
            self._subscriptions_merge_dialog.set_feed(feed)
            return 1
        else:
            return 0

    def display_item_unselected(self, item):
        if item and item is not self._visible_item:
            if item.feed is not self._visible_feed:
                straw.main_window.display_feed_unselected(item.feed)
            straw.main_window.display_item_unselected(item)

    def display_item(self, item):
        if item and item is not self._visible_item:
            if item.feed is not self._visible_feed:
                straw.main_window.display_feed_unselected(item.feed)
            self._visible_item = item
            item.seen = 1
            straw.main_window.display_item(item)

    def get_next_item(self, item = None):
        if item is None:
            item = self._visible_item
        if item is None:
            if not len(straw.config.feeds):
                return None
            feed = straw.config.feeds[0]
            return feed.get_first_item()
        feed = item.feed
        ni = feed.get_next_item(item)
        if ni is not None:
            return ni
        else:
            feed_idx = straw.config.feeds.index(feed)
            fi = 1
            while fi <= len(straw.config.feeds):
                item = straw.config.feeds[
                    (feed_idx + fi) % len(straw.config.feeds)].get_first_item()
                if item is not None:
                    return item
                fi += 1
            return None

    def display_next_item(self, item = None):
        self.display_item_unselected(self.get_next_item(item))

    def display_next_unread_item(self, item = None):
        current = self._visible_item
        while 1:
            item = self.get_next_item(item)
            # so we can check if we have gone around even if the user didn't
            # have anything selected
            if current is None:
                current = item
            if item is None:
                return
            if not item.seen:
                self.display_item_unselected(item)
                return
            if item is current:
                return

    def get_previous_item(self, item = None):
        if item is None:
            item = self._visible_item
        if item is None:
            if not len(straw.config.feeds):
                return None
            feed = straw.config.feeds[-1]
            return feed.get_last_item()
        feed = item.feed
        ni = feed.get_previous_item(item)
        if ni is not None:
            return ni
        else:
            feed_idx = straw.config.feeds.index(feed)
            fd = 1
            while fd <= len(straw.config.feeds):
                item = straw.config.feeds[
                    (feed_idx - fd) % len(straw.config.feeds)].get_last_item()
                if item is not None:
                    return item
                fd += 1
            return None

    def display_previous_item(self, item = None):
        self.display_item_unselected(self.get_previous_item(item))

    # called periodically to display any messages the poller thread
    # has requested to be shown
    def display_messages(self, *a):
        for m in self._user_messages:
            straw.main_window.display_status_message(m)
        self._user_messages = []
        gtk.timeout_add(1000, self.display_messages)

    def display_status_message(self, m):
        straw.main_window.display_status_message(m)

    def is_visible_item(self, item):
        return item is self._visible_item

    def show_preferences_dialog(self):
        self._preferences_dialog.show()

    def show_subscribe_dialog(self, url=None):
        self._subscribe_dialog.show(url)

    def show_find_dialog(self):
        self._find_dialog.show()

    def show_subscriptions_merge_dialog(self):
        self._subscriptions_merge_dialog.show(self._visible_feed)
        self.disable_feed_selection()

    def show_export_subscriptions_dialog(self):
        self._export_subscriptions_dialog.show()

    def show_import_subscriptions_dialog(self):
        self._import_subscriptions_dialog.show()

    def enable_feed_selection(self):
        straw.main_window.enable_feed_selection()

    def disable_feed_selection(self):
        straw.main_window.disable_feed_selection()

    def mainloop(self):
        gtk.threads_init()
        gtk.main()

    def quit(self):
        gtk.mainquit()
        self._itemstore.stop()
        return

    def show_image(self, widget, image):
        self._image_loader.show(widget, image)

    def import_opml(self, filename):
        opml = straw.OPMLImport.read(open(filename))
        if opml is not None:
            current_feeds = {}
            for feed in straw.config.feeds:
                current_feeds[feed.access_info[0]] = 1
            for b in opml:
                if current_feeds.has_key(b.xmlurl):
                    continue
                straw.config.feeds.append(straw.Feed(b.title, b.xmlurl))
        return


class ImageLoader:
    def __init__(self, libdir):
        self.libdir = libdir

    def show(self, widget, image):
        fn = os.path.join(self.libdir, image + ".png")
        widget.set_from_file(fn)

def find_lib_dir():
    if os.environ.has_key("STRAW_DATA"):
        return os.environ["STRAW_DATA"]
    h, t = os.path.split(os.path.split(os.path.abspath(sys.argv[0]))[0])
    if t == 'bin':
        fp = os.path.join(h, 'share', 'straw')
        if os.path.isdir(fp):
            return fp
        fp = os.path.join(h, 'lib', 'straw')
        if os.path.isdir(fp):
            return fp
    raise "FileNotFoundError", "couldn't find Straw data directory"

def find_locale_dir():
    h, t = os.path.split(os.path.split(os.path.abspath(sys.argv[0]))[0])
    if t == 'bin':
        fp = os.path.join(h, 'share', 'locale')
        if os.path.isdir(fp):
            return fp
    raise "FileNotFoundError", "couldn't find locale data directory"

def find_glade_file(libdir):
    xmlfile = None
    if os.environ.has_key("STRAW_GLADE"):
        return os.environ["STRAW_GLADE"]
    fp = os.path.join(libdir, 'straw.glade')
    if os.path.isfile(fp):
        return fp
    raise "FileNotFoundError", "couldn't find straw.glade"

