cvs_id = "$Id: PollManager.py,v 1.15 2003/03/16 10:32:01 juri Exp $"

import straw
import time
import gtk
from error import log
from urllib import quote as urlquote
import traceback
import socket

class PollManager:
    def __init__(self):
        self._lastpoll = -1
        self._poll_frequency = straw.config.poll_frequency
        straw.config.signal_connect(straw.PollFrequencyChangedSignal,
                              self.poll_freq_changed)
        gtk.timeout_add(10000, self.maybe_poll)
        
    def poll_freq_changed(self, signal):
        self._poll_frequency = signal.sender.poll_frequency

    def maybe_poll(self):
        now = time.time()
        if (self._poll_frequency > 0) and (now - self._lastpoll > self._poll_frequency):
            self._lastpoll = now
            self.poll()
        gtk.timeout_add(10000, self.maybe_poll)

    def poll(self, feed = None):
        if straw.config.offline:
            return
        straw.main.display_status_message(_("Starting polling"))
        if feed:
            feeds = [feed]
        else:
            feeds = straw.config.feeds
        for f in feeds:
            straw.main.display_status_message(
                _("Polling %s...") % f.title)
            p = Poller(f)
            p.poll()

class ExtrasConsumer:
    def __init__(self, feed, field):
        self.feed = feed
        self.field = field

    def http_results(self, status, headers, data):
        if status[1] == 200:
            setattr(self.feed, self.field, data)

    def http_failed(self, exception):
        pass

class Poller:
    _parser = straw.SummaryParser()
    _parsermp = straw.SummaryParserMP()
    
    def __init__(self, feed):
        self._feed = feed

    def poll(self):
        url, user, pw = self._feed.get_access_info()
        parsed = None
        if url is None or url == "":
            return None

        headers = {}
        if self._feed.previous_etag is not None:
            headers['If-None-Match'] = self._feed.previous_etag
        try:
            pc = straw.URLFetch.connection_manager.request(
                url, self, headers, user, pw,
                priority=straw.NetworkConstants.PRIORITY_RSS)
        except Exception, e:
            self.http_failed(e)

    def http_results(self, status, header, data):
        success = self.update_feed(status, header, data)
        if success:
            straw.main.display_status_message(
                _("Parsing data of %s done.") % self._feed.title)
        else:
            straw.main.display_status_message(
                _("Parsing data of %s failed.") % self._feed.title)
        self.update_feed_extras()
        self._feed.poll_done()

    def update_feed_extras(self):
        try:
            for i in self._feed.items:
                for name in i.image_keys():
                    if not i.get_image(name):
                        image = straw.ImageCache.retrieve(name, 1, headers = {'Referer': self._feed.get_access_info()[0]})
                        i.set_image(name, image)
        except Exception, ex:
            # if we don't get the images, we can probably still show
            # the text
            pass
        if self._feed.channel_blogroll_location:
            bc = ExtrasConsumer(self._feed, 'channel_blogroll')
            straw.URLFetch.connection_manager.request(
                self._feed.channel_blogroll_location, bc)
        if self._feed.channel_subscriptions_location:
            sc = ExtrasConsumer(self._feed, 'channel_subscriptions')
            straw.URLFetch.connection_manager.request(
                self._feed.channel_subscriptions_location, sc)

    def update_feed(self, status, header, data):
        straw.main.display_status_message(
            _("Parsing data for feed %s...") % self._feed.title)
        if status[1] == 304:
            self._feed.error = None
            return 1
        elif status[1] != 200:
            self._feed.error = _("Polling feed resulted in abnormal status '%s' (code %d)") % (status[2], status[1])
            return None
        parsed = None
        self._feed.invalid_rss = 0
        try:
            parsed = self._parser.parse(data, self._feed)
        except straw.RSSParseException, e:
            parsed = self._parsermp.parse(data, self._feed)
            self._feed.invalid_rss = 1
        except Exception, e:
            self._feed.error = _("Unknown exception when parsing RSS: %s") % str(e)
            return None
        etag = header.getheader('etag')
        if etag is not None:
            self._feed.previous_etag = etag
        self._feed.channel_title = parsed.title
        self._feed.channel_description = parsed.description
        self._feed.channel_copyright = parsed.copyright
        self._feed.channel_link = parsed.link
        self._feed.channel_subscriptions_location = parsed.subscriptions
        self._feed.channel_blogroll_location = parsed.blogroll
        self._feed.channel_blink = parsed.blink
        parsed.items.reverse()
        self._feed.add_items(parsed.items)
        self._feed.error = None
        return 1

    def http_failed(self, exception):
        if isinstance(exception, socket.error):
            err = exception.args[1]
        else:
            err = str(exception)
        self._feed.error = err
        straw.main.display_status_message(
            _("Polling of feed %s failed") % self._feed.title)
        self._feed.poll_done()

class RSSParseException(Exception):
    pass
