cvs_id = "$Id: SubscribeDialog.py,v 1.14 2003/02/23 14:55:04 juri Exp $"

import gobject
import gtk
from types import FunctionType
import traceback

import straw
from error import log

class SubscribeDialog:
    buttons_visible = {
        'subscribe_dialog_start':    [0, 1, 1, 1, 1, 1],
        'subscribe_dialog_previous': [0, lambda s: len(s._pagestack) > 1,
                                      0, 0, 0, 0],
        'subscribe_dialog_next':     [0, 0, 1, 0, 1, 0],
        'subscribe_dialog_finish':   [0, 1, 0, 0, 0, 0]}

    COLUMN_URL = 0
    COLUMN_TITLE = 1
    COLUMN_DESCRIPTION = 2
    COLUMN_FEED = 3
    COLUMN_PARSEDSUMMARY = 4

    PAGE_START = 0
    PAGE_FINISH = 1
    PAGE_PROBLEM = 2
    PAGE_NOTFOUND = 3
    PAGE_SELECT = 4
    PAGE_WAIT = 5
    
    def __init__(self, xml):
        self._window = xml.get_widget("subscribe_dialog")
        self._window.set_transient_for(straw.main_window.get_window())
        self._page = -1
        self._pagestack = []
        nameFuncMap = {}
        for key in dir(self.__class__):
            if key[:3] == 'on_':
                nameFuncMap[key] = getattr(self, key)
        xml.signal_autoconnect(nameFuncMap)
        for widget in ['subscribe_site_entry', 
                       'new_feed_name_entry', 'subscribe_notebook',
                       'feed_error_display', 'subscribe_dialog_start',
                       'subscribe_dialog_finish', 'subscribe_dialog_next',
                       'subscribe_dialog_previous',
                       'subscribe_feed_found_url',
                       'subscribe_feed_found_description',
                       'subscribe_multiple_match_view']:
            self.__dict__['_%s' % widget] = xml.get_widget(widget)
        model = gtk.ListStore(
            gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,
            gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)
        self._subscribe_multiple_match_view.set_model(model)
        self.create_columns(self._subscribe_multiple_match_view)

    def show(self, url):
        self.show_buttons_for_page(self.PAGE_START)
        self._window.show()
        self._window.present()
        if url:
            self.find_site(url)

    def hide(self):
        self._new_feed = None
        self.set_page(self.PAGE_START, reset = 1)
        self._subscribe_site_entry.set_text("")
        self._window.hide()

    def create_columns(self, treeview):
        for cname, cnum in [('URL', self.COLUMN_URL),
                            (_('Title'), self.COLUMN_TITLE),
                            (_('Description'), self.COLUMN_DESCRIPTION)]:
            renderer = gtk.CellRendererText()
            column = gtk.TreeViewColumn(cname, renderer, text = cnum)
            treeview.append_column(column)

    def show_buttons_for_page(self, page):
        for name, visible in self.buttons_visible.items():
            widget = self.__dict__['_%s' % name]
            if type(visible[page]) == FunctionType:
                val = visible[page](self)
            else:
                val = visible[page]
            if val:
                widget.show()
            else:
                widget.hide()
        return

    def set_page(self, page, reset = 0, nostack = 0):
        self._subscribe_notebook.set_current_page(page)
        self.show_buttons_for_page(page)
        if not reset:
            self._pagestack.append(self._page)
        else:
            self._pagestack = []
        self._page = page

    def display_selected_feed(self, feed, ps):
        self._new_feed_name_entry.set_text(feed.title)
        self._subscribe_feed_found_url.set_text(feed.access_info[0])
        self._subscribe_feed_found_description.set_text(
            feed.channel_description)
        self.set_page(self.PAGE_FINISH)
        self._new_feed = (feed, ps)
        return

#     def get_url(self, url):
#         feed = straw.Feed(
#             "temporary feed", url)
#         ps = straw.poll_manager.poll(feed)
#         try:
#             pass
#         except IOError, ex:
#             self.set_page(self.PAGE_NOTFOUND)
#             return
#         except Exception, ex:
#             self.set_page(self.PAGE_PROBLEM)
#             self._feed_error_display.set_text(str(ex))
#         feed.title = ps.title
#         feed.channel_description = ps.description
#         if len(ps.items) == 0:
#             self.set_page(self.PAGE_PROBLEM)
#             self._feed_error_display.set_text(
#                 _("There were no items in this feed"))
#             self._new_feed = (feed, None)
#             return
#         self.display_selected_feed(feed, ps)
#         return
        
    def parse_feed(self, data, feed):
        try:
            parsed = straw.SummaryParser().parse(data, feed)
        except straw.RSSParseException, e:
            parsed = straw.SummaryParserMP().parse(data, feed)
            parsed.invalid_rss = 1
        except Exception, e:
            log("parse_feed: caught exception ", e)
            traceback.print_exc()
            return None
        return parsed

    def display_feed_choices(self, feeds):
        model = self._subscribe_multiple_match_view.get_model()
        model.clear()
        for feed, ps in feeds:
            iter = model.append()
            model.set(iter, self.COLUMN_URL, feed.access_info[0],
                      self.COLUMN_TITLE, feed.title,
                      self.COLUMN_DESCRIPTION, feed.channel_description,
                      self.COLUMN_FEED, feed, self.COLUMN_PARSEDSUMMARY, ps)
        self.set_page(self.PAGE_SELECT)

    def find_site(self, site):
        self.set_page(self.PAGE_WAIT, nostack = 1)
        while gtk.events_pending():
            gtk.mainiteration(gtk.FALSE)
        try:
            feeds = straw.rssfinder.getFeeds(site)
        except:
            self.set_page(self.PAGE_NOTFOUND)
            return
        if len(feeds) == 0:
            self.set_page(self.PAGE_NOTFOUND)
            return
        if len(feeds) == 1:
            feed = straw.Feed("temporary feed", feeds[0][0])
            ps = self.parse_feed(feeds[0][1], feed)
            feed.title = ps.title
            feed.channel_description = ps.description
            self.display_selected_feed(feed, ps)
        else:
            dfeeds = []
            for url, data in feeds:
                feed = straw.Feed("temporary feed", url)
                ps = self.parse_feed(data, feed)
                feed.title = ps.title
                feed.channel_description = ps.description
                dfeeds.append((feed, ps))
            self.display_feed_choices(dfeeds)

    def on_subscribe_find_site_clicked(self, button, *args):
        self.find_site(self._subscribe_site_entry.get_text())
        
    def on_subscribe_dialog_start_clicked(self, button, *args):
        self.set_page(self.PAGE_START)

    def on_subscribe_dialog_next_clicked(self, button, *args):
        if self._page == self.PAGE_SELECT:
            selection = self._subscribe_multiple_match_view.get_selection()
            model, iter = selection.get_selected()
            index = model.get_path(iter)[0]
            f = model[index][self.COLUMN_FEED]
            ps = model[index][self.COLUMN_PARSEDSUMMARY]
            self.display_selected_feed(f, ps)
        if self._page == self.PAGE_PROBLEM:
            self.display_selected_feed(
                self._new_feed[0], self._new_feed[1], None)

    def on_subscribe_dialog_finish_clicked(self, button, *args):
        feed = self._new_feed[0]
        ps = self._new_feed[1]
        feed.title = self._new_feed_name_entry.get_text()
        feed.channel_title = ps.title
        feed.channel_description = ps.description
        feed.channel_copyright = ps.copyright
        feed.channel_link = ps.link
        feed.channel_subscriptions = ps.subscriptions
        feed.channel_blogroll = ps.blogroll
        feed.channel_blink = ps.blink
        straw.config.feeds.append(feed)
        ps.items.reverse()
        feed.add_items(ps.items)
        feed.invalid_rss = ps.invalid_rss
        poller = straw.Poller(feed)
        poller.update_feed_extras()
        feed.poll_done()
        self.hide()

    def on_subscribe_dialog_previous_clicked(self, button, *args):
        page = self._pagestack.pop()
        self._subscribe_notebook.set_current_page(page)
        self.show_buttons_for_page(page)
        self._page = page

    def on_subscribe_dialog_cancel_clicked(self, button, *args):
        self.hide()

    def on_subscribe_site_entry_key_press_event(self, widget, event):
        if event.keyval == gtk.keysyms.Return:
            self.find_site(self._subscribe_site_entry.get_text())

    def on_subscribe_feed_url_entry_key_press_event(self, widget, event):
        if event.keyval == gtk.keysyms.Return:
            self.find_rss()

    def on_subscribe_dialog_delete_event(self, *args):
        self.hide()
        return 1
