cvs_id = "$Id: SubscriptionsMergeDialog.py,v 1.8 2003/03/01 11:16:43 juri Exp $"

import gobject
import gtk
import gnome

import straw

class FeedsList:
    COLUMN_NAME = 0
    COLUMN_OBJECT = 1

    def __init__(self, widget, dialog):
        self._widget = widget
        self._dialog = dialog
        widget.get_selection().connect("changed", self.feed_selection_changed)
        self.initialize_treeview(self._widget)
    
    def feed_selection_changed(self, *args):
        selection = self._widget.get_selection()
        s = selection.get_selected()
        if s:
            model, iter = s
            if iter is None:
                return
            index = model.get_path(iter)[0]
            self.blog_selected(model[index][self.COLUMN_OBJECT])

    def initialize_treeview(self, widget):
        model = gtk.ListStore(str, gobject.TYPE_PYOBJECT)
        widget.set_model(model)
        for cname, cnum in ((_('Name'), self.COLUMN_NAME),):
            renderer = gtk.CellRendererText()
            column = gtk.TreeViewColumn(cname, renderer, text = cnum)
            widget.append_column(column)

    def blog_selected(self, blog_id):
        self._dialog.blog_selected(blog_id)

    def set_blogs(self, blogs):
        model = self._widget.get_model()
        model.clear()
        for b in blogs:
            iter = model.append()
            model.set(iter, self.COLUMN_NAME, b.text, self.COLUMN_OBJECT, b)

class SubscriptionsMergeDialog:
    PAGE_MAIN = 0
    PAGE_WAIT = 1

    def __init__(self, xml):
        self._window = xml.get_widget('subscriptions_merge_dialog')
        self._window.set_transient_for(straw.main_window.get_window())
        self._fl = FeedsList(xml.get_widget('subscriptions_new_treeview'),
                             self)
        self._description_label = xml.get_widget(
            'subscriptions_selected_description')
        self._url_label = xml.get_widget('subscriptions_selected_url')
        self._feed_name = xml.get_widget('subscriptions_feed_name')
        self._popup_subscriptions = xml.get_widget(
            'subscriptions_popup_subscriptions')
        self._popup_blogroll = xml.get_widget('subscriptions_popup_blogroll')
        self._subscribe_button = xml.get_widget('subscriptions_subscribe')
        self._unsubscribe_button = xml.get_widget('subscriptions_unsubscribe')
        self._notebook = xml.get_widget('subscriptions_merge_notebook')
        self._selected_blog = None
        nameFuncMap = {}
        for key in dir(self.__class__):
            if key[:3] == 'on_':
                nameFuncMap[key] = getattr(self, key)
        xml.signal_autoconnect(nameFuncMap)

    def show(self, feed):
        self._window.show()
        self._window.present()
        self._notebook.set_current_page(self.PAGE_WAIT)
        while gtk.events_pending():
            gtk.mainiteration(gtk.FALSE)
        subscriptions = feed.channel_subscriptions
        blogroll = feed.channel_blogroll
        self._notebook.set_current_page(self.PAGE_MAIN)
        self._popup_subscriptions.set_sensitive(subscriptions is not None)
        self._popup_blogroll.set_sensitive(blogroll is not None)
        self._blogroll_list = blogroll
        self._subscriptions_list = subscriptions
        self._subscribe_button.set_sensitive(0)
        if subscriptions is not None:
            self.display_list(subscriptions)
        elif blogroll is not None:
            self.display_list(blogroll)

    def hide(self):
        self._window.hide()
        straw.main.enable_feed_selection()

    def blog_selected(self, blog):
        self._selected_blog = blog
        self._url_label.set_text(blog.url)
        self._description_label.set_text(
            (blog.description, blog.text)[blog.description is None])
        self._subscribe_button.set_sensitive(1)
            
    def on_subscriptions_close_clicked(self, *args):
        self.hide()

    def set_feed(self, feed):
        self._feed = feed
        self._feed_name.set_text(feed.title)

    def on_subscriptions_popup_blogroll_activate(self, *args):
        self.display_list(self._blogroll_list)

    def on_subscriptions_popup_subscriptions_activate(self, *args):
        self.display_list(self._subscriptions_list)

    def on_subscriptions_subscribe_clicked(self, *args):
        if self._selected_blog.xmlurl is not None:
            f = straw.Feed(self._selected_blog.text,
                           location = self._selected_blog.xmlurl)
            straw.config.feeds.append(f)
        else:
            straw.main.show_subscribe_dialog(self._selected_blog.url)

    def on_subscriptions_browse_site_clicked(self, *args):
        gnome.url_show(self._url_label.get_text())

    def display_list(self, bloglist):
        self._fl.set_blogs(bloglist)
