# Copyright (c) 2002 Juri Pakaste
# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

cvs_id = "$Id: SummaryItem.py,v 1.8 2003/02/20 15:58:30 juri Exp $"

import straw
from mx.DateTime import now

class SummaryItem(object, straw.SignalEmitter):
    __slots__ = ('title', 'link', 'description', 'guid', 'pub_date', 'source',
                 '_images', '_seen', '_id', 'feed', '_slots',
                 'fm_license', 'fm_changes', 'creator', 'license_urls',
                 '_searchable_fields', '_sticky')

    _searchable_fields = ('title', 'description', 'fm_license', 'fm_changes',
                          'creator')

    def __init__(self):
        straw.SignalEmitter.__init__(self)
        self.initialize_slots(straw.ItemReadSignal, straw.ItemStickySignal)
        self.title = None
        self.link = None
        self.description = None
        self.guid = None
        self.pub_date = now()
        self.source = None
        self._images = {}
        self._seen = 0
        self._id = None
        self.feed = None
        self.fm_license = None
        self.fm_changes = None
        self.creator = None
        self.license_urls = []
        self._sticky = 0

    def set_image(self, image_name, image):
        self._images[image_name] = image
        
    def get_image(self, image_name):
        return self._images.get(image_name, None)

    def clear_images(self):
        self._images = {}

    def image_keys(self):
        return self._images.keys()

    def is_seen(self):
        return self._seen

    def set_seen(self, seen = 1):
        oldseen = self._seen
        self._seen = seen
        if seen and not oldseen:
            self.emit_signal(straw.ItemReadSignal(self))

    seen = property(is_seen, set_seen, None, "")

    def is_sticky(self):
        return self._sticky

    def set_sticky(self, sticky):
        self._sticky = sticky
        self.emit_signal(straw.ItemStickySignal(self))

    sticky = property(is_sticky, set_sticky, None, "")

    def set_seen_quiet(self, seen = 1):
        old = self._seen
        self._seen = seen
        return seen != old

    def set_id(self, id):
        self._id = id

    def get_id(self):
        return self._id

    id = property(get_id, set_id, None, "Internal id")

    def match(self, text):
        def try_field(field):
            return type(field) in (
                type(''), type(u'')) and field.lower().find(text) > -1
        for f in self._searchable_fields:
            try:
                if try_field(self.__getattribute__(f)):
                    return 1
            except AttributeError:
                pass
        return 0
            
