# Copyright (c) 2002 Juri Pakaste
# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

cvs_id = "$Id: SummaryParser.py,v 1.17 2003/03/14 22:38:55 juri Exp $"

from xml.sax import saxutils, make_parser, SAXParseException
from xml.sax.handler import feature_namespaces, feature_namespace_prefixes
from StringIO import StringIO
import xml.sax._exceptions
import rfc822
import time
import urlparse
import traceback
from mx import DateTime
from mx.DateTime.Parser import DateTimeFromString
import SummaryParserCommon
import straw
from error import log

# various xml namespaces we use
BLOGCHANNEL_NS = "http://backend.userland.com/blogChannelModule"
CONTENT_NS = "http://purl.org/rss/1.0/modules/content/"
DUBLINCORE_NS = "http://purl.org/dc/elements/1.1/"
FRESHMEAT_NS = "http://freshmeat.net/rss/fm/"
RSS1_NS = "http://purl.org/rss/1.0/"
RSS2_NS = "http://backend.userland.com/rss2"
RSS09_NS = "http://my.netscape.com/rdf/simple/0.9/"
RDFCC_NS = "http://web.resource.org/cc/"
RSSCC_NS = "http://backend.userland.com/creativeCommonsRssModule"
RDF_NS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"

class RSSHandler(saxutils.DefaultHandler):
    def __init__(self, feed):
        self._summary = straw.ParsedSummary()
        self._feed = feed
        self._content = ""
        self._attrs = {}
        self._in_item = 0
        self._in_textinput = 0
        self._in_description = 0
        self._item = None
        self._element = None
        self._default_license_urls = []

    def startElementNS(self, (uri, name), qname, attrs):
        if not self._in_item and name == 'item':
            self._in_item = 1
            self._item = straw.SummaryItem()
        elif name == 'textinput':
            self._in_textinput = 1
        elif self._in_item and name == 'description':
            self._in_description = 1
        elif self._in_description:
            raise ValueError, 'No elements allowed in description'
        self._content = ""
        self._attrs = attrs

    def endElementNS(self, (uri, name), qname):
        if self._in_item:
            if uri in (None, RSS09_NS, RSS1_NS, RSS2_NS):
                if name == 'title':
                    self._item.title = self._content.strip()
                elif name == 'description':
                    self._in_description = 0
                    if self._item.description is None:
                        self._item.description = self._content.strip()
                elif name == 'link':
                    self._item.link = self._content.strip()
                elif name == 'guid':
                    self._item.guid = self._content.strip()
                elif name == 'pubDate':
                    try:
                        self._item.pub_date = DateTimeFromString(
                            self._content.strip(), ('lit',))
                    except Exception, e:
                        self._item.pub_date = None
                elif name == 'source':
                    self._item.source = {'url': self._attrs[(None, u'url')],
                                         'text': self._content.strip()}
                elif name == 'item':
                    self._in_item = 0
                    if self._item.description is not None:
                        idh = SummaryParserCommon.ItemDescriptionHandler(self._feed)
                        try:
                            idh.feed(self._item.description)
                            images = idh.get_images()
                            for i in images:
                                self._item.set_image(i, None)
                            description = idh.get_data()
                            self._item.description = description
                        except Exception, ex:
                            pass
                    if self._item.license_urls == []:
                        self._item.license_urls = self._default_license_urls[:]
                    self._summary.addItem(self._item)
                    self._item = None
            elif uri == FRESHMEAT_NS:
                if name == 'license':
                    self._item.fm_license = self._content.strip()
                elif name == 'changes':
                    self._item.fm_changes = self._content.strip()
            elif uri == CONTENT_NS:
                if name == 'encoded':
                    self._item.description = self._content.strip()
            elif uri == DUBLINCORE_NS:
                if name == 'date':
                    try:
                        self._item.pub_date = DateTimeFromString(
                            self._content.strip(), ('iso',))
                    except:
                        self._item.pub_date = DateTime.now()
                elif name == 'creator':
                    self._item.creator = self._content.strip()
            elif uri == RDFCC_NS:
                if name == "license":
                    u = self._attrs[(RDF_NS, 'resource')]
                    if u is not None:
                        self._item.license_urls.append(u)
            elif uri == RSSCC_NS:
                if name == "license":
                    self._item.license_urls.append(self._content.strip())

        elif self._in_textinput:
            if name == 'textinput':
                self._in_textinput = 0
        else:
            # channel head
            if uri in (None, RSS09_NS, RSS1_NS, RSS2_NS):
                if name == 'title':
                    self._summary.title = self._content.strip()
                elif name == 'link':
                    self._summary.link = self._content.strip()
                elif name == 'description':
                    self._summary.description = self._content.strip()
                elif name == 'language':
                    self._summary.language = self._content.strip()
                elif name == 'lastBuildDate':
                    try:
                        self._summary.last_build_date = DateTimeFromString(
                            self._content.strip(), ('iso',))
                    except:
                        self._summary.last_build_date = DateTime.now()
                elif name == 'copyright':
                    self._summary.copyright = self._content.strip()
            elif uri == DUBLINCORE_NS:
                if name == 'date':
                    try:
                        self._summary.last_build_date = DateTimeFromString(
                            self._content.strip(), ('iso',))
                    except:
                        self._summary.last_build_date = DateTime.now()
                elif name == 'rights':
                    self._summary.copyright = self._content.strip()
            elif uri == BLOGCHANNEL_NS:
                if name == 'blogRoll':
                    self._summary.blogroll = self._content.strip()
                elif name == 'mySubscriptions':
                    self._summary.subscriptions = self._content.strip()
                elif name == 'blink':
                    self._summary.blink = self._content.strip()
            elif uri == RDFCC_NS:
                if name == "license":
                    u = self._attrs[(RDF_NS, 'resource')]
                    if u is not None:
                        self._default_license_urls.append(u)
            elif uri == RSSCC_NS:
                if name == "license":
                    self._default_license_urls.append(self._content.strip())

    def characters(self, ch):
        self._content += ch
        return

    def get_summary(self):
        return self._summary

class SummaryParser:
    def __init__(self):
        pass

    def parse(self, content, feed):
        parser = make_parser()
        parser.setFeature(feature_namespaces, 1)
        handler = RSSHandler(feed)
        parser.setContentHandler(handler)

        enc = "utf-8"
        try:
            tenc = SummaryParserCommon.read_encoding(content)
            if tenc:
                enc = tenc
        except ValueError:
            pass
        try:
            parser.parse(StringIO(content))
        except SAXParseException, ex:
            raise straw.RSSParseException, "Invalid RSS"
        except ValueError, ex:
            raise straw.RSSParseException, "Invalid RSS"

        parsed = handler.get_summary()
        for item in parsed.items:
            item.feed = feed
        return parsed
