cvs_id = "$Id: SummaryParserMP.py,v 1.12 2003/03/14 22:38:55 juri Exp $"

from mx import DateTime
from mx.DateTime.Parser import DateTimeFromString
import SummaryParserCommon
import straw
import re

iso_date_re = re.compile('^\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d')

class SummaryParserMP:
    def __init__(self):
        pass

    def parse(self, content, feed):
        parser = straw.RSSParser()
        enc = None
        try:
            enc = SummaryParserCommon.read_encoding(content)
            if enc is None:
                raise ValueError
        except ValueError:
            pass
        if not enc:
            enc = "utf-8"
        parser.feed(content)
        parsed = straw.ParsedSummary()

        def unicode_field(dict, key, default = None, enc = enc):
            v = dict.get(key, default)
            if v is not None:
                try:
                    v = unicode(v, enc)
                except UnicodeError:
                    # try with iso-8859-1, usually we get at least something
                    v = unicode(v, "iso-8859-1")
                    enc = "iso-8859-1"
                v = v.strip()
            return (v, enc)

        def read_date(s):
            s = s.strip()
            try:
                if iso_date_re.match(s):
                    d = DateTimeFromString(s, ('iso',))
                else:
                    d = DateTimeFromString(s, ('lit',))
            except ValueError:
                d = DateTime.now()
            return d

        parsed.title, enc = unicode_field(
            parser.channel, "title", "", enc = enc)
        parsed.description, enc = unicode_field(
            parser.channel, "description", "", enc = enc)
        parsed.link, enc = unicode_field(parser.channel, "link", "", enc = enc)
        parsed.copyright, enc = unicode_field(
            parser.channel, "rights", "", enc = enc)
        parsed.last_build_date = read_date(parser.channel.get("date", ""))

        for idict in parser.items:
            item = straw.SummaryItem()
            item.title, enc = unicode_field(idict, 'title', enc = enc)
            item.link = idict.get('link', None)
            if item.link:
                item.link = item.link.strip()
            item.description, enc = unicode_field(idict, 'description', enc = enc)
            ce, enc = unicode_field(idict, 'content_encoded', enc = enc)
            if ce is not None:
                item.description = ce
            item.guid, enc = unicode_field(idict, 'guid', enc = enc)
            item.pub_date = read_date(idict.get('date', ""))
            if idict.has_key('source_url'):
                st, enc = unicode_field(idict, 'source', '', enc = enc)
                item.source = {'url': idict['source_url'].strip(), 'text': st}
            if item.description is not None:
                idh = SummaryParserCommon.ItemDescriptionHandler(feed)
                try:
                    idh.feed(item.description)
                    images = idh.get_images()
                    for i in images:
                        item.set_image(i, None)
                except Exception, ex:
                    pass
            item.feed = feed
            parsed.addItem(item)
        return parsed

